/*
 * DataClass.cpp
 *
 *  Created on: Mar 19, 2010
 *      Author: roger
 */

#include "DataClass.h"

#define _POSIX_SOURCE 1 	// POSIX compliant source (??)
#define FALSE 0
#define TRUE 1

/* Centers packet reception so it always starts with '10' char */
void DataClass::recenter_transmission()
{
    /* Setting necessary port parameters */
    struct termios oldtio,newtio;	// port settings
    tcgetattr(fd,&oldtio); 		// save current port settings
    unsigned char buf1[1];		// buffer for received data
    unsigned char buf2[nb_char-1];
    bool ready = true;

    tcgetattr(fd,&oldtio); 		// save current port settings
    tcgetattr(fd,&newtio);              // directly use original settings (except for size of data packets to be read)

    newtio.c_cc[VTIME]    = 0;          // inter-character timer unused
    newtio.c_cc[VMIN]     = 1;          // blocking read until 1 chars received
    tcflush(fd, TCIFLUSH);
    tcsetattr(fd,TCSANOW,&newtio);

    /* looking for next '10' char */
    printf("\nRECENTERING ...");
    while (ready)
    {
        read(fd,buf1,1);
        if ((unsigned short)buf1[0]==10)
        {
            ready = false;
        }
        printf(".");
    } 
    printf("starting 'char' found : ");

    /* setting new port parameters */
    newtio.c_cc[VTIME]    = 0;          // inter-character timer unused
    newtio.c_cc[VMIN]     = nb_char-1;        // blocking read until nb_char chars received
    tcflush(fd, TCIFLUSH);
    tcsetattr(fd,TCSANOW,&newtio);

    read(fd,buf2,nb_char-1);
    printf("postion corrected\n");

    /* resetting old port parameters */
    tcsetattr(fd,TCSANOW,&oldtio);

//    delete[] buf1;
//    delete[] buf2;
}

/* Constructor:
 *  string aFile : port reference
 */
DataClass::DataClass(string aFile)
{
    /* variable definitions */
    inputfile = aFile;
    nb_l = NBL;
    nb_c = NBC;
    matrixFilter = new int[nb_l*nb_c];
    rawMat = new double[nb_l*nb_c];
    null_buf = new unsigned char[nb_char];
    for(int ij=0;ij<nb_char;ij++)
    {
        null_buf[ij]=200;
    }

    /* setting port parameters */
    struct termios port_settings;               // structure to store the port settings in
    fd = open(inputfile.c_str(), O_RDWR | O_NOCTTY );
    if (fd <0)
    {
        perror(inputfile.c_str());
        throw 1;
    }
    fcntl(fd, F_SETFL);
    tcgetattr(fd, &port_settings);
    cfsetispeed(&port_settings, B115200);        // set baud rates
    cfsetospeed(&port_settings, B115200);
    port_settings.c_cflag |= (CLOCAL | CREAD);
    port_settings.c_cflag &= ~PARENB;            // set no parity, stop bits, data bits
    port_settings.c_cflag &= ~CSTOPB;
    port_settings.c_cflag &= ~CSIZE;
    port_settings.c_cflag |= CS8;
    port_settings.c_cflag &= ~CRTSCTS;
    port_settings.c_cc[VMIN]     = nb_char;
    port_settings.c_cc[VTIME]    = 0;
    tcflush(fd, TCIFLUSH);
    tcsetattr(fd, TCSANOW, &port_settings);     // apply the settings to the port

    write(fd,sensibility,1);                    // send sensibility parameters to finger

    /* 'initializes' transmisssion as for '10' char to be at packet beginning */
    recenter_transmission();

    initTime();

    if(saveRAWmat)
    {
        savefile.open(saveUnMATfile);
    }
}

/* Destructor */
DataClass::~DataClass()
{
    if(fd>=0)
    {
        close(fd);

        for(int i=0;i<data.size();i++)
        {
            for(int j=0;j<data[i].size();j++)
            {
                data[i][j].clear();
            }
            data[i].clear();
        }
        data.clear();
    }
    delete []matrixFilter;
    if(savefile.is_open())
    {
        savefile.close();
    }
    delete null_buf;
    delete[] rawMat;
}

/* Closes file stream (to serial port) */
void DataClass::Close()
{
    close(fd);                      //closing link
}

/* Removes oldest data matrix from memory */
void DataClass::removeMatrix()
{
    data.erase(data.begin());
    timedata.erase(timedata.begin());
}

/* Reads data from port and saves it in memory as matrix */
void DataClass::readMatrix()
{
    int i1=0;
    int i2=0;
    int buf;
//    unsigned char s_buf[nb_char];
    unsigned char *s_buf = new unsigned char[nb_char];
    int res;
    int ct = 1;

    /* reads data from port */
    res = read(fd,s_buf,nb_char);           // returns after nb_char chars have been input
    s_buf[res]=0;

    if(!doRecord)
    {
        s_buf = null_buf;
    }

    if (doRecord && (unsigned short)s_buf[0]!=10)       // if data packet valid
    {
        recenter_transmission();
    }
    else                                    // else: if data packet is OK
    {
        getMaxFilter(s_buf);                // adapts filter depending on data read

        vector<vector<double> > tmp_data;   // temporary variables
        vector<double> buf1;
        tmp_data.push_back(buf1);

        while (i1<nb_l)                     // filling of matrix with data
        {
            buf = normalizeData(s_buf[ct],ct-1);
            if (ct>=nb_char)                      // if end of packet (should not occure)
            {
                std::cout<<"\nERROR : line 105 DATACLASS.CPP\n"<<std::endl;
                return;
            }
            ct++;

            tmp_data[i1].push_back(buf);    // copying first line of matrix
            i2++;
            if (i2>=nb_c)
            {
                i1++;
                i2=0;
                tmp_data.push_back(buf1);
            }
        }

        data.push_back(tmp_data);           // adding matrix to 'long term' memory
    }
//    delete[] s_buf;
    boost::posix_time::time_duration Dtime = boost::posix_time::ptime(boost::posix_time::microsec_clock::universal_time()) - startTIME;

    timedata.push_back(Dtime.total_milliseconds());

    saveINvar(s_buf);
    saveINfile(s_buf,timedata.back());

}

/* returns an element of oldest matrix in memory */
double DataClass::getMat(int i,int j)
{
    return data[0][i][j];
}

/* returns number of matrixes in memory */
int DataClass::getNbMat()
{
    return data.size();
}

/* applies simple filter to data matrix (old version)*/
int DataClass::normalizeData(unsigned char s_buf)
{
    int tmp = (unsigned short) s_buf;
    //    printf("%hd ::",tmp);
    //return abs(200 - tmp);
    int atemp = PAD_sleep_state - PAD_sleep_state*static_threshold/100;

    if (tmp < atemp)
        return (int)PAD_sleep_state - (int)tmp;
    else
        return 0;
}

/* applies defined filter to data matrix */
int DataClass::normalizeData(unsigned char s_buf,int index)
{
    int tmp = (unsigned short) s_buf;
    int atemp = PAD_sleep_state - PAD_sleep_state*static_threshold/100;    // 5% of normal state of macanoreceptor

    if (matrixFilter[index]!=0 && tmp < atemp)
        return (int)PAD_sleep_state - (int)tmp;
    else
        return 0;
}

/* builds matrix filter depending on values aqcuired by port */
void DataClass::getMaxFilter(unsigned char *s_buf)
{
    /* extracts max value mesured */
    int max=0;
    float d_threshold = dynamic_threshold /100;
    int t_max;
    for(int i=1;i<nb_l*nb_c+1;i++)
    {
        matrixFilter[i-1] = 0;                                  // resets old version of filter to 0
        if(max<PAD_sleep_state-(unsigned short)s_buf[i])
            max = PAD_sleep_state-(unsigned short)s_buf[i];
    }
    if(max > PAD_sleep_state*static_threshold/100)              // if data has been acquired
    {
        for(int i=1;i<nb_l*nb_c+1;i++)                          // finding mecanoreceptors with values > max/2
        {
            if(max*d_threshold<PAD_sleep_state-(unsigned short)s_buf[i])  // if value > max*%
            {
                t_max = PAD_sleep_state-(unsigned short)s_buf[i];
                matrixFilter[i-1] = 2;
                for(int k=nb_c-1;k<=nb_c+1;k++)                 // looking at the values of the 8 mecanoreceptors around it
                {
                    if(i-1-k>=0 && t_max*d_threshold<PAD_sleep_state-(unsigned short)s_buf[i-k])
                    {
                        matrixFilter[i-k-1]=1;
                    }
                    if(i-1+k<nb_c*nb_l && t_max*d_threshold<PAD_sleep_state-(unsigned short)s_buf[i+k])
                    {
                        matrixFilter[i+k-1]=1;
                    }
                }
                if(i-1>/*=*/0 && t_max*d_threshold<PAD_sleep_state-(unsigned short)s_buf[i-1])
                {
                    matrixFilter[i-1-1]=1;
                }
                if(i+1<nb_c*nb_l && t_max*d_threshold<PAD_sleep_state-(unsigned short)s_buf[i+1])
                {
                    matrixFilter[i+1-1]=1;
                }
            }
        }
    }
}

int DataClass::getTime()
{
    return timedata.front();
}

void DataClass::initTime()
{
    startTIME = boost::posix_time::ptime(boost::posix_time::microsec_clock::universal_time());   // gets start time of program
}

int DataClass::getINTdata(unsigned char s_buf)
{
    int tmp = (unsigned short) s_buf;
    return (int)PAD_sleep_state - (int)tmp;
}

void DataClass::saveINfile(unsigned char * s_buf, int time)
{
    bool tOK = false;
    saveDataOKSemaphore->acquire(1);
    tOK = saveDataOK;
    saveDataOKSemaphore->release(1);
    if(tOK && saveRAWmat)
    {
        for(int i=0;i<nb_c*nb_l;i++)
        {
            savefile << getINTdata(s_buf[i+1]) << "   ";
        }
        savefile << time << endl;
        savefile.flush();
    }
}

void DataClass::saveINvar(unsigned char *s_buf)
{
        for(int i=0;i<nb_c*nb_l;i++)
        {
            rawMat[i] = (double)normalizeData(s_buf[i+1]);
        }
}
