#include "DataSIMClass.h"

DataSIMClass::DataSIMClass()
{
    index_counter = -1;
    nb_l = NBL;
    nb_c = NBC;
    matrixFilter = new int[nb_l*nb_c];

    if(saveRAWmat)
    {
        savefile.open(saveUnMATfile);
    }
    rawMat = new double[nb_l*nb_c];
}

DataSIMClass::~DataSIMClass() {
    delete[] matrixFilter;
    for(int i=0;i<data.size();i++)
    {
        delete data[i];
    }
    data.clear();
    if(savefile.is_open())
    {
        savefile.close();
    }
    delete[] rawMat;
}

void DataSIMClass::removeMatrix()
{
    data.erase(data.begin());
    timedata.erase(timedata.begin());
}


void DataSIMClass::listen2socket()
{
    try
    {
        fingerMAT.clean();

        string senderAddress = SimulatorAddress;
        string myAddress = ThisAddressS;
        unsigned short recPort = Socket::resolveService(RecFingerSIMPort,"udp");

        UDPSocket rec_sock;
        rec_sock.setLocalAddressAndPort(myAddress, recPort);
        int recvMsgSize = rec_sock.recvFrom(&fingerMAT, sizeof(fingerMAT), senderAddress, recPort);
    }
    catch (SocketException &e)
    {
        cout << e.what() << endl;
        return;
    }
}


void DataSIMClass::readMatrix()
{
    listen2socket();

    int buf;
    int *tmp_tab = new int[nb_l*nb_c];
    double *tab = new double[nb_l*nb_c];
    for(int w=0;w<nb_l*nb_c;w++)
    {
        tmp_tab[w]= (int)fingerMAT.data[w];
    }
    getMaxFilter(tmp_tab);

    // filling of data with desired matrix
    int counter = 0;
    while(counter<nb_l*nb_c)
    {
        if(useFILEfilter == 1)
        {
            buf = normalizeData(tmp_tab[counter],counter);
        }
        else
        {
            buf = tmp_tab[counter];
        }
        tab[counter] = buf;
        counter ++;
    }
    data.push_back(tab);
//    printf(/*"timeFLAG: %f, */"TIME : %f\n",/*fingerMAT.timestamp,*/fingerMAT.time);
    timedata.push_back((int)fingerMAT.time);

    saveINfile(tmp_tab,timedata.back());
    saveINvar(tmp_tab);

    delete[] tmp_tab;
}

double DataSIMClass::getMat(int i,int j)
{
    //	return data[0][i][j];
    return data[0][i*nb_c+j];
}

int DataSIMClass::getNbMat()
{
    return data.size();
}

/* applies defined filter to data matrix */
int DataSIMClass::normalizeData(int s_buf,int index)
{
    int tmp = s_buf;
    int atemp = PAD_sleep_state*static_threshold/100;    // 5% of normal state of macanoreceptor

    if (matrixFilter[index]!=0 && tmp > atemp)
        return tmp;
    else
        return 0;
}

/* applies static filter to data matrix */
int DataSIMClass::normalizeData(int s_buf)
{
    int tmp = s_buf;
    int atemp = PAD_sleep_state*static_threshold/100;    // 5% of normal state of macanoreceptor

    if (tmp > atemp)
        return tmp;
    else
        return 0;
}

/* builds matrix filter depending on values aqcuired by port */
void DataSIMClass::getMaxFilter(int *s_buf)
{
    /* extracts max value mesured */
    int max=0;
    float d_threshold = dynamic_threshold /100;
    int t_max;
    for(int i=0;i<nb_l*nb_c;i++)
    {
        matrixFilter[i] = 0;                                  // resets old version of filter to 0
        if(max<s_buf[i])
            max = s_buf[i];
    }
    if(max > PAD_sleep_state*static_threshold/100)              // if data has been acquired
    {
        for(int i=0;i<nb_l*nb_c;i++)                          // finding mecanoreceptors with values > max/2
        {
            if(max*d_threshold<s_buf[i])  // if value > max/2
            {
                t_max = s_buf[i];
                matrixFilter[i] = 2;
                for(int k=nb_c-1;k<=nb_c+1;k++)                 // looking at the values of the 8 mecanoreceptors around it
                {
                    if(i-k>=0 && t_max*d_threshold<s_buf[i-k])
                    {
                        matrixFilter[i-k]=1;
                    }
                    if(i+k<nb_c*nb_l && t_max*d_threshold<s_buf[i+k])
                    {
                        matrixFilter[i+k]=1;
                    }
                }
                if(i-1>=0 && t_max*d_threshold<s_buf[i-1])
                {
                    matrixFilter[i-1]=1;
                }
                if(i+1<nb_c*nb_l && t_max*d_threshold<s_buf[i+1])
                {
                    matrixFilter[i+1]=1;
                }
            }
        }
    }
}

int DataSIMClass::getTime()
{
    return timedata.front();
}

void DataSIMClass::saveINfile(int * s_buf, int time)
{
    bool tOK = false;
    saveDataOKSemaphore->acquire(1);
    tOK = saveDataOK;
    saveDataOKSemaphore->release(1);
    if(tOK && saveRAWmat)
    {
        for(int i=0;i<nb_c*nb_l;i++)
        {
            savefile << s_buf[i] << "   ";
        }
        savefile << time << endl;
        savefile.flush();
    }
}

void DataSIMClass::saveINvar(int *s_buf)
{
        for(int i=0;i<nb_c*nb_l;i++)
        {
            rawMat[i] = (double)normalizeData(s_buf[i]);
        }
}
