#ifndef DATAIMCLASS_H
#define DATAIMCLASS_H

#include <string>
#include <vector>
#include <iostream>
#include <fstream>

#include "parameters.hpp"
#include <boost/date_time/posix_time/posix_time.hpp>
#include "PracticalSocket.h"
#include "globalVar.h"

#define matlabAddress "192.168.178.25"
#define thisAddress "192.168.178.27"
#define recFmatlabPORT "25252"

using namespace std;


// struct of commands to be sent to robot
struct fingerMatType
{
    fingerMatType()
    {
        clean();
    };
    void clean()
    {
        timestamp=0;
        for(int i=0;i<24;i++)
        {
            data[i] = 0;
        }
        time = 0;
    }
    double timestamp;       // to check for loss of information
    double data[24];        // data matrix (24 values)
    double time;
};



class DataSIMClass
{
public:
    DataSIMClass();
    ~DataSIMClass();

protected:
        int nb_c,nb_l;                          //Matrix dimensions
        vector<double*> data;
        int index_counter;
        int *matrixFilter;                      // matrix filter filled by 'getMaxFilter'
        vector<int> timedata;                   // vector of time values
        fingerMatType fingerMAT;
        ofstream savefile;                      // save file for un-pre-processed mat data
public:
        double * rawMat;                        // saves raw Mat for display


public:
        void listen2socket();
        void removeMatrix();	//removes a matrix vector
        void readMatrix();		//reads a matrix from the file
        double getMat(int i,int j);	//returns data(i,j) of the oldest matrix
        int getNbMat();			// returns the number of matrices currently in memory
        int normalizeData(int s_buf,int index);
        void getMaxFilter(int* s_buf);// fills in a matrix filter to reduce noise
        int getTime();                          // returns oldest data time
        void saveINfile(int* s_buf,int time);
        void saveINvar(int* s_buf);   // saves raw mat in var for display
        int normalizeData(int s_buf);
};

#endif // DATAIMCLASS_H
