#ifndef CEREBELLUMCOMM_H
#define CEREBELLUMCOMM_H

#include <QThread>
#include <QList>
#include "parameters.hpp"
#include "utils/cd_socket.hpp"
#include <fstream>
#include <stdio.h>
#include <stdlib.h>
#include "PracticalSocket.h"

struct cerebellumCommType       // struct to communicate (both ways) with cerebellum interface
{
    cerebellumCommType()
    {
        clean();
    }
    void clean()
    {
        time = -1;
        statusflag = 0;
        posX = 0;
        posY = 0;
        vX = 0;
        vY = 0;
        aX = 0;
        aY = 0;
    }
    double time;            // to check for loss of information + sync (in s)
    double statusflag;      // 0=wait | 1=initial positioning | 2=move | -1=reset | -2=endline
    double posX;            // in m
    double posY;            // in s
    double vX;              // in m/s
    double vY;              // in m/s
    double aX;              // acceleration on x axis (in m/s)
    double aY;              // acceleration on y axis (in m/s)
};


struct fingerSimCmdType         // struct to send commands to fingerSim
{
    fingerSimCmdType()
    {
        clean();
    }
    void clean()
    {
        time = -1;
        posX = 0;
        posY = 0;
    }
    double time;        // to check for loss of information (in s)
    double posX;        // in m
    double posY;        // in m
};




class CerebellumComm : public QThread
{
    Q_OBJECT
public:
    CerebellumComm();
    ~CerebellumComm();
    void Send2cerebelum();                      // sends info/cmd to cerebellum
    cerebellumCommType getLastCommand();        // get last relevant command
    cerebellumCommType RecFromCerebellum();     // wait for data reception from Cerebellum
    void Send2fingerSim(cerebellumCommType CMD);// send info/cmd to fingerSim
    void clean();                               // clears the cmd vector
    virtual void run();
    void initSocketTCP();                       // initializes connection with cerebellum

    QList<cerebellumCommType> CommVect; // store cerebellum commands to be sent (adds connection delays)
    double currentTime;                 // saves current time (given by cerebellum)
    double lastSimCmdTime;              // Last time a command was sent to fingerSim
    Cd_Socket * CerebellumSocket;       // Socket for Communication with cerebellum interface

signals:
    void updateArmStateInfo(cerebellumCommType);     // signal to update stored arm state (if useful for Joker)

public slots:
    void add2fifo(double posX,double posY,double vX,double vY,double aX,double aY,double robotFlag);       // adds command to cerebellum command fifo list
    void OperationCompleteFeedbackReceived();                                                               // for process thread to inform when it is over with calculations (to let cerebellum continue sim)
};



#endif // CEREBELLUMCOMM_H
