#include "edlut.h"

QSemaphore *StoreDisplayEDLUTSemaphore= new QSemaphore(1);
QVector<int> *DisplayEDLUTSpikeData = new QVector<int>;
QVector<int> *DisplayEDLUTTimeData = new QVector<int>;


//Sen2Matlab * MATLAB = new Sen2Matlab();

EDLUT::EDLUT()
{
    nb_spikes = 0;
    SimulationTime=100000000;
    last_timebb = 0;
    lastSIMULtime = 0;

    Simul = NULL;
    OutputDriver = NULL;
    InputDriver = NULL;

    saveData = false;

    Sproc = new seperateLetters();

    if(intParser("useCN"))
    {
        OutputDriver = new ArrayOutputSpikeDriver();
        InputDriver = new ArrayInputSpikeDriver();
        std::string netTMPp(charParser(CN_NET).c_str());
        std::string wghTMPp(charParser(CN_WGH).c_str());
        const char* EdNet = netTMPp.c_str();
        const char* EdWgh = wghTMPp.c_str();
        Simul = new Simulation(EdNet/*charParser(CN_NET).c_str()*/,EdWgh/*charParser(CN_WGH).c_str()*/, SimulationTime);
        Simul->AddInputSpikeDriver(InputDriver);
        Simul->AddOutputSpikeDriver(OutputDriver);
        try
        {
            Simul->InitSimulation();
        }
        catch(EDLUTException e)
        {
            perror("simulation initialisation");
        }
    }
    //!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    MATLAB = new Sen2Matlab();
    //!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
}


EDLUT::~EDLUT()
{
    if(OUTPUTS.is_open())
        OUTPUTS.close();
    if(Simul!=NULL)
        Simul->EndSimulation();

    try
    {
        delete Simul;
    }
    catch(exception e)
    {
        perror("error deleting Simul");
    }
    delete InputDriver;
    try
    {
        delete OutputDriver;
    }
    catch(exception e)
    {
        perror("error deleting OutputDriver");
    }

    InputSpikeTimes.clear();
    InputSpikeCells.clear();
}


void EDLUT::AddData(vector<int> & data,int& time)
{
    if(useCN)
    {
        for(int i=0;i<data[0];i++)
        {
            InputSpikeTimes.push_back(((double)time/1000) + lastSIMULtime + (((double)EDLUTtimeDelay)/1000));       // added Time delay !!!!
            InputSpikeCells.push_back(data[i+1]);
            nb_spikes++;
        }
    }
}

void EDLUT::AddDataForSorting(vector<int> & data,int& time)
{
    if(useCN)
    {
        Sproc->addnewSpikes(data,time);
    }
}

void EDLUT::addSortedData(/*QList<int> spikeCells,QList<double> spikeTimes*/)
{
//    QList<int> spikeCells = Sproc->spikeNeuronsOUT;
//    QList<double> spikeTimes = Sproc->spikeTimesOUT;
//    QList<int>::iterator cellIt = spikeCells.begin();
//    QList<double>::iterator timesIt = spikeTimes.begin();

    QList<int>::iterator cellIt = Sproc->spikeNeuronsOUT.begin();
    QList<double>::iterator timesIt = Sproc->spikeTimesOUT.begin();

    for(cellIt = Sproc->spikeNeuronsOUT.begin()/*spikeCells.begin()*/;cellIt!=Sproc->spikeNeuronsOUT.end()/*spikeCells.end()*/;cellIt++)
    {
        InputSpikeTimes.push_back((*timesIt/1000) + lastSIMULtime);
        InputSpikeCells.push_back(*cellIt);
//        printf("EDLUT spikeTimes(SECs) : %f || spikeTimes(ms): %f\n",InputSpikeTimes.last(),*timesIt);
        timesIt++;
        nb_spikes++;
    }
    Sproc->spikeNeuronsOUT.clear();
    Sproc->spikeTimesOUT.clear();
}

void EDLUT::runLetterSeperation()
{
    Sproc->mainfunc();
    addSortedData();
//    cout << InputSpikeTimes.size() << endl;
}



void EDLUT::RunSimul()
{
    if(useCN)
    {
        if(nb_spikes>0)
        {
//            for(int i=0;i<nb_spikes;i++)
//            {
//                if(InputSpikeTimes[i]<last_timebb)
//                {
//                    InputSpikeTimes[i]=(double)last_timebb;
//                    puts("Changed Time");
//                }
//            }

//                Simul->InitSimulation();

            double endtime = InputSpikeTimes.last();
            last_timebb = endtime;

            double * OutputSpikeTimes;
            long int * OutputSpikeCells;
//            cout << Simul->GetTotalSimulationTime() << endl;
            InputDriver->LoadInputs(Simul->GetQueue(),Simul->GetNetwork(),nb_spikes,InputSpikeTimes.data(),InputSpikeCells.data());
            Simul->RunSimulationSlot(endtime);

            int OutputNumber = OutputDriver->GetBufferedSpikes(OutputSpikeTimes,OutputSpikeCells);


            if(useSPIKEdisplay==1)
            {
                StoreDisplayEDLUTSemaphore->acquire(1);                      // releasing memory space
            }
            for (int i=0; i< OutputNumber; ++i)
            {
                if(saveData && saveCNspikes)
                {
                    OUTPUTS  << (int)((OutputSpikeTimes[i]*1000-lastSIMULtime*1000)) << "\t" << OutputSpikeCells[i]-24 << endl;
                }
                if(useSPIKEdisplay==1)
                {
                    DisplayEDLUTSpikeData->push_back((int)(OutputSpikeCells[i]-24));
                    DisplayEDLUTTimeData->push_back((int)((OutputSpikeTimes[i]*1000-lastSIMULtime*1000)));
                }
            }
            if(useSPIKEdisplay==1)
            {
                StoreDisplayEDLUTSemaphore->release(1);                      // releasing memory space
            }
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
            MATLAB->computeSpikeCounts(OutputNumber,OutputSpikeCells);
            //MATLAB->Send();
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
            if(OutputNumber>0)
            {
                delete [] OutputSpikeTimes;
                delete [] OutputSpikeCells;
            }
            //        else
            //        {
            //            delete OutputSpikeTimes;
            //            delete OutputSpikeCells;
            //        }

            //Simul->StopSimulation();
            InputSpikeTimes.clear();
            InputSpikeCells.clear();
            nb_spikes = 0;
//            last_timebb = endtime;
        }
//        Simul->PrintInfo(cout);
    }
}

void EDLUT::STOP()
{
    if(useCN)
    {
        lastSIMULtime = last_timebb;
        if(OUTPUTS.is_open())
            OUTPUTS.close();
//        if(Simul!=NULL)                       //removed because apparently useless
//            Simul->StopSimulation();

        //    try
        //    {
        //        delete Simul;
        //    }
        //    catch(exception e)
        //    {
        //        perror("error deleting Simul");
        //    }
        //    Simul = NULL;
        //    delete InputDriver;
        //    InputDriver = NULL;
        //    try
        //    {
        //        delete OutputDriver;
        //    }
        //    catch(exception e)
        //    {
        //        perror("error deleting OutputDriver");
        //    }
        //    OutputDriver=NULL;

        InputSpikeTimes.clear();
        InputSpikeCells.clear();
    }
}

void EDLUT::START()
{
    if(useCN)
    {
        nb_spikes = 0;
        SimulationTime=100000000;
        last_timebb = 0;

        Sproc->reinit();    //////////////////////////////////////////////////////////////////////////////////////////////////////

        //    OutputDriver = new ArrayOutputSpikeDriver();
        //    InputDriver = new ArrayInputSpikeDriver();
        //    Simul = new Simulation(NetworkFile,WeightsFile, SimulationTime);
        //    Simul->AddInputSpikeDriver(InputDriver);
        //    Simul->AddOutputSpikeDriver(OutputDriver);
//        try
//        {
//            Simul->InitSimulation();                                  // removed because apparently useless
//        }
//        catch(EDLUTException e)
//        {
//            perror("simulation initialisation");
//        }

        if(saveCNspikes)
        {
            OUTPUTS.open(saveCNfile/*outputFILE.c_str()*/);
            if (!OUTPUTS.is_open())
            {
                std::cout << "Error opening data file !" << std::endl;
            }
        }
    }
}
