//( firingneuron.cpp


#include <math.h>


#include "firingneuron.hpp"
#include <iostream>
#include <stdio.h>
#include <assert.h>

int FiringNeuron::PresynapticConnection::numberOfConnection = 0;


//( FiringNeuron

FiringNeuron::FiringNeuron()
{
  positionIndex = 0;
  noiseFlag = false;
}


FiringNeuron::~FiringNeuron()
{
  for (PresynapticConnectionList::const_iterator iterator = presynapticConnections.begin();
       iterator != presynapticConnections.end();
       iterator++)
    {
      delete (*iterator);
    }
}


void FiringNeuron::setPositionIndex(double positionIndex)
{
  this->positionIndex = positionIndex;
}


double FiringNeuron::getPositionIndex() const
{
  return positionIndex;
}


void FiringNeuron::setNoise(bool noiseFlag)
{
  this->noiseFlag = noiseFlag;
}


void FiringNeuron::addPresynapticConnection(Synapse * synapse, double weight)
{
  assert(synapse != 0);
  presynapticConnections.push_back(new PresynapticConnection(synapse, weight));
}


void FiringNeuron::generateSpike()
{
  for (PresynapticConnectionList::const_iterator iterator = presynapticConnections.begin();
       iterator != presynapticConnections.end();
        iterator++)
    {
      sendSpike((*iterator)->synapse, (*iterator)->weight);
    }
}

void FiringNeuron::sendSpike(Synapse * synapse, double weight)
{
  synapse->sendSpike(weight);
}


void FiringNeuron::setStimulus(const double intensity)
{
  assert(false);
}


void FiringNeuron::printWeights(ofstream & weightsRaster)
{
  double counter = 0;
  for(PresynapticConnectionList::const_iterator iterator = presynapticConnections.begin();
      iterator!=presynapticConnections.end();
      iterator++)
    {
      weightsRaster << counter << "    " << (*iterator)->weight << endl;
      counter+=1;
    }
}


double FiringNeuron::getWeight(int connectionIndex)
{
  assert((unsigned)connectionIndex<presynapticConnections.size());
  int counter = 0;
  for(PresynapticConnectionList::const_iterator iterator = presynapticConnections.begin();
      iterator!=presynapticConnections.end();
      iterator++)
    {
      if(counter == connectionIndex) return (*iterator)->weight;
      counter ++;
    }
  return -100;
}


void FiringNeuron::printAllWeights(ofstream & weightsStream)
{
//  PresynapticConnectionList::const_iterator iterator = presynapticConnections.begin();
////    cout << "presynaptic connection size : " << presynapticConnections.size() << endl;
//  for(int index=0; index<numberOfPyramidalCell; index ++)
//    {
//      weightsStream << (*iterator)->weight << "\t";
//      iterator ++;
//    }

    for(PresynapticConnectionList::const_iterator iterator = presynapticConnections.begin();
      iterator!=presynapticConnections.end();
      iterator++)
    {
      weightsStream << (*iterator)->weight << "\t";
    }
}

void FiringNeuron::initPostSynapticFirings(FiringPositions & efferentFiringPositions)
{
  this->efferentFiringPositions = & efferentFiringPositions;
}

//) FiringNeuron


//) firingneuron.cpp
