#include "initpos.h"

initPos::initPos(string aFile)
{
    /* variable definitions */
    inputfile = aFile;
    nb_l = NBL;
    nb_c = NBC;
    matrixFilter = new int[nb_l*nb_c];

    timeStamp = 0;

    ASensitivity = 5;
    refIndx = 21;            // to be changed according to position of letter
    pitchIndx = 19;
    rollIndx = 15;
    yawIndx = 13;

    pdirect = 1;
    rdirect = 1;
    rspeed_ref = 0.05;
    rspeed = rspeed_ref;
    rspeed_step = 0.01;


    /* setting port parameters */
    struct termios port_settings;               // structure to store the port settings in
    fd = open(inputfile.c_str(), O_RDWR | O_NOCTTY );
    if (fd <0)
    {
        perror(inputfile.c_str());
        throw 1;
    }
    fcntl(fd, F_SETFL);
    tcgetattr(fd, &port_settings);
    cfsetispeed(&port_settings, B115200);        // set baud rates
    cfsetospeed(&port_settings, B115200);
    port_settings.c_cflag |= (CLOCAL | CREAD);
    port_settings.c_cflag &= ~PARENB;            // set no parity, stop bits, data bits
    port_settings.c_cflag &= ~CSTOPB;
    port_settings.c_cflag &= ~CSIZE;
    port_settings.c_cflag |= CS8;
    port_settings.c_cflag &= ~CRTSCTS;
    port_settings.c_cc[VMIN]     = nb_char;
    port_settings.c_cc[VTIME]    = 0;
    tcflush(fd, TCIFLUSH);
    tcsetattr(fd, TCSANOW, &port_settings);     // apply the settings to the port

    write(fd,sensibility,1);                    // send sensibility parameters to finger

    /* 'initializes' transmisssion as for '10' char to be at packet beginning */
    recenter_transmission();


}

void initPos::recenter_transmission()
{
    /* Setting necessary port parameters */
    struct termios oldtio,newtio;	// port settings
    tcgetattr(fd,&oldtio); 		// save current port settings
    unsigned char buf1[1];		// buffer for received data
    unsigned char buf2[nb_char-1];
    bool ready = true;

    tcgetattr(fd,&oldtio); 		// save current port settings
    tcgetattr(fd,&newtio);              // directly use original settings (except for size of data packets to be read)

    newtio.c_cc[VTIME]    = 0;          // inter-character timer unused
    newtio.c_cc[VMIN]     = 1;          // blocking read until 1 chars received
    tcflush(fd, TCIFLUSH);
    tcsetattr(fd,TCSANOW,&newtio);

    /* looking for next '10' char */
    printf("\nRECENTERING ...");
    while (ready)
    {
        read(fd,buf1,1);
        if ((unsigned short)buf1[0]==10)
        {
            ready = false;
        }
        printf(".");
    }
    printf("starting 'char' found : ");

    /* setting new port parameters */
    newtio.c_cc[VTIME]    = 0;          // inter-character timer unused
    newtio.c_cc[VMIN]     = nb_char-1;        // blocking read until nb_char chars received
    tcflush(fd, TCIFLUSH);
    tcsetattr(fd,TCSANOW,&newtio);

    read(fd,buf2,nb_char-1);
    printf("postion corrected\n");

    /* resetting old port parameters */
    tcsetattr(fd,TCSANOW,&oldtio);

    //    delete[] buf1;
    //    delete[] buf2;
}

/* Closes file stream (to serial port) */
void initPos::Close()
{
    close(fd);                      //closing link
}

/* Reads data from port and saves it in memory as matrix */
void initPos::readMatrix()
{
    int i1=0;
    int i2=0;
    int buf;
    unsigned char s_buf[nb_char];
    int res;
    int ct = 1;

    /* reads data from port */
    res = read(fd,s_buf,nb_char);           // returns after nb_char chars have been input
    s_buf[res]=0;
    if ((unsigned short)s_buf[0]!=10)       // if data packet valid
    {
        recenter_transmission();
    }
    else                                    // else: if data packet is OK
    {
        getMaxFilter(s_buf);                // adapts filter depending on data read

        //QVector< QVector<double> > tmp_data;   // temporary variables
        QVector<double> buf1;

        while (i1<nb_l)                     // filling of matrix with data
        {
            buf = normalizeData(s_buf[ct],ct-1);
            if (ct>=nb_char)                      // if end of packet (should not occure)
            {
                std::cout<<"\nERROR : line 105 DATACLASS.CPP\n"<<std::endl;
                return;
            }
            ct++;

            buf1.push_back(buf);    // copying first line of matrix
            i2++;
            if (i2>=nb_c)
            {
                i1++;
                i2=0;
                //tmp_data.push_back(buf1);
            }
        }
        //        !!!!!!!!!!!!!!!!!!!!!!
        data.clear();                       // empties Data vector
        data = buf1;           // adding matrix to 'long term' memory
    }
    //    delete[] s_buf;

}

/* applies simple filter to data matrix (old version)*/
int initPos::normalizeData(unsigned char s_buf)
{
    int tmp = (unsigned short) s_buf;
    int atemp = PAD_sleep_state - PAD_sleep_state*static_threshold/100;

    if (tmp < atemp)
        return (int)PAD_sleep_state - (int)tmp;
    else
        return 0;
}

/* applies defined filter to data matrix */
int initPos::normalizeData(unsigned char s_buf,int index)
{
    int tmp = (unsigned short) s_buf;
    int atemp = PAD_sleep_state - PAD_sleep_state*static_threshold/100;    // 5% of normal state of macanoreceptor

    if (matrixFilter[index]!=0 && tmp < atemp)
        return (int)PAD_sleep_state - (int)tmp;
    else
        return 0;
}

/* builds matrix filter depending on values aqcuired by port */
void initPos::getMaxFilter(unsigned char *s_buf)
{
    /* extracts max value mesured */
    int max=0;
    float d_threshold = dynamic_threshold /100;
    int t_max;
    for(int i=1;i<nb_l*nb_c+1;i++)
    {
        matrixFilter[i-1] = 0;                                  // resets old version of filter to 0
        if(max<PAD_sleep_state-(unsigned short)s_buf[i])
            max = PAD_sleep_state-(unsigned short)s_buf[i];
    }
    if(max > PAD_sleep_state*static_threshold/100)              // if data has been acquired
    {
        for(int i=1;i<nb_l*nb_c+1;i++)                          // finding mecanoreceptors with values > max/2
        {
            if(max*d_threshold<PAD_sleep_state-(unsigned short)s_buf[i])  // if value > max/2
            {
                t_max = PAD_sleep_state-(unsigned short)s_buf[i];
                matrixFilter[i-1] = 2;
                for(int k=nb_c-1;k<=nb_c+1;k++)                 // looking at the values of the 8 mecanoreceptors around it
                {
                    if(i-1-k>=0 && t_max*d_threshold<PAD_sleep_state-(unsigned short)s_buf[i-k])
                    {
                        matrixFilter[i-k-1]=1;
                    }
                    if(i-1+k<nb_c*nb_l && t_max*d_threshold<PAD_sleep_state-(unsigned short)s_buf[i+k])
                    {
                        matrixFilter[i+k-1]=1;
                    }
                }
                if(i-1>/*=*/0 && t_max*d_threshold<PAD_sleep_state-(unsigned short)s_buf[i-1])
                {
                    matrixFilter[i-1-1]=1;
                }
                if(i+1<nb_c*nb_l && t_max*d_threshold<PAD_sleep_state-(unsigned short)s_buf[i+1])
                {
                    matrixFilter[i+1-1]=1;
                }
            }
        }
    }
}

int initPos::getYawMvt(int counter,int maxCount)
{
    if(counter >= maxCount)
    {
        pdirect = - pdirect;
        printf("YAW control - limit of exploration space reached\n");
        return 2;
    }

    if(data.size()==nb_c*nb_l)
    {
        int Amax = (int)data[refIndx];
        if(abs(Amax - (int)data[yawIndx])>ASensitivity)
        {
            if(abs(Amax - (int)data[yawIndx])>abs(Amax - (int)olddata[yawIndx]))
            {
                rspeed = rspeed - rspeed_step;
                pdirect = - pdirect;
                printf("YAW control - changing rotation direction\n");
            }
            return 0;
        }
        else
            return 1;
    }
    return 0;
}

int initPos::getPitchMvt()
{
    if(data.size()==nb_c*nb_l)
    {
        int Amax = (int)data[refIndx];
        if(abs(Amax - (int)data[pitchIndx])>ASensitivity)
        {
            if(abs(Amax - (int)data[pitchIndx])>abs(Amax - (int)olddata[pitchIndx]))
            {
                rspeed = rspeed - rspeed_step;
                pdirect = - pdirect;
                printf("PITCH control - changing rotation direction\n");
            }
            return 0;
        }
        else
            return 1;
    }
    return 0;
}

int initPos::getRollMvt()
{
    if(data.size()==nb_c*nb_l)
    {
        int Amax = (int)data[refIndx];
        if(abs(Amax - (int)data[rollIndx])>ASensitivity)
        {
            if(abs(Amax - (int)data[rollIndx])>abs(Amax - (int)olddata[rollIndx]))
            {
                rspeed = rspeed - rspeed_step;
                rdirect = - rdirect;
                printf("ROLL control - changing rotation direction\n");
            }
            return 0;
        }
        else
        {
            return 1;
        }
    }
    return 0;
}

int initPos::Send(int index)
{
    timeStamp++;
    sendInitInfo MSG;

    MSG.timeStamp = timeStamp;
    MSG.statusFlag = 1;

    if(index == 1)      // ie YAW
    {
        MSG.wz = rspeed;
    }
    else if(index == 2) // ie PITCH
    {
        MSG.wx = rspeed;
    }
    else if(index == 3) // ie ROLL
    {
        MSG.wy = rspeed;
    }

    string servAddress = robotAddress;//robotIP;
    unsigned short echoServPort = Socket::resolveService(SendRobotPort/*robotPort*/,"udp");
    try
    {
        UDPSocket sock;
        sock.sendTo(&MSG, sizeof(MSG), servAddress, echoServPort);
    }
    catch (SocketException &e)
    {
        cerr << e.what() << endl;
        return 1;
    }
    cout << "Initial Positionning Command Sent.\n";

    return 0;
}

void initPos::SwapData()
{
    olddata.clear();
    for(int i=0;i<data.size();i++)
    {
        olddata.push_back(data.at(i));
    }
}

void initPos::run()
{
    printf("INITPOS : initializing ...");
    while(olddata.size()<24)
    {
        printf(" ... ");
        readMatrix();
        SwapData();
    }
    printf("\n");


    int tmp = 0;
    int count = 0;
    int maxCounter = 100;
    printf("INITPOS : correcting Yaw :\n");
    while (tmp == 0)
    {
        readMatrix();
        tmp = getYawMvt(count, maxCounter);
        if(tmp == 2)
        {
            count = 0;
            maxCounter = 2*maxCounter;
            tmp = 0;
        }
        Send(1);
    }
    rspeed = rspeed_ref;
    tmp = 0;

    printf("INITPOS : correcting Pitch :\n");
    while (tmp == 0)
    {
        readMatrix();
        tmp = getPitchMvt();
        Send(2);
    }
    rspeed = rspeed_ref;
    tmp = 0;

    printf("INITPOS : correcting Roll :\n");
    while(tmp == 0)
    {
        readMatrix();
        tmp = getRollMvt();
        Send(3);
    }
    rspeed = rspeed_ref;
}


int initPos::listen2socket()
{
    try
    {
        string senderAddress = robotAddress;
        string myAddress = ThisAddressR;
        unsigned short recPort = Socket::resolveService(RecRobotPort,"udp");

        UDPSocket rec_sock;
        rec_sock.setLocalAddressAndPort(myAddress, recPort);
        int recvMsgSize = rec_sock.recvFrom(&RSP, sizeof(RSP), senderAddress, recPort);
    }
    catch (SocketException &e)
    {
        cout << e.what() << endl;
        return 1;
    }

    return 0;
}

int initPos::initialExploration()
{
    QVector <int> PADvalue;         // value measured by ref sensor PAD
    QVector <double> PADxPos;       // position of finger on X axis
    QVector <double> PADyPos;       // position of finger on Y axis

    double hl = 0.002;              // half diameter of zone to explore (in meters)

    listen2socket();

    double initXvalue = RSP.xpos;
    double initYvalue = RSP.ypos;

    double x_minlimit = initXvalue - hl;
    double x_maxlimit = initXvalue + hl;
    double y_minlimit = initYvalue - hl;
    double y_maxlimit = initYvalue + hl;

    //placing the finger at the top left corner of the region
    //!!!!!Send Position to x_minlinit and y_maxlimit to robot (depends on axis of robot)

    // starting exploration
    double x_obj = x_maxlimit;
    double y_obj = y_maxlimit;
    int x_dir = 1;              // direction of movement on xAxis
    int cnt = 1;
    double oldtimestamp = -1;
    while(cnt == 1)
    {
        listen2socket();                    // not cool, socket already occupied (use already existing receiveFrobot-> sharedRSP)
        if(oldtimestamp<RSP.timestamp)
        {
            oldtimestamp = RSP.timestamp;
            if(RSP.xpos>=x_maxlimit)
            {
                x_dir = -1;     // change direction -> go backwards
                //+ go to next line (ypos = ypos+1)??
            }
            if(RSP.xpos<=x_minlimit)
            {
                x_dir = 1;      // go forward
                //+go to next line ?
            }
            readMatrix();
            PADvalue.push_back(data[refIndx]);         // value measured by ref sensor PAD
            PADxPos.push_back(RSP.xpos);       // position of finger on X axis
            PADyPos.push_back(RSP.ypos);       // position of finger on Y axis
        }
    }
}
