#ifndef INITPOS_H
#define INITPOS_H

#include <string>
#include <vector>
#include <iostream>
#include <fstream>
#include <termios.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <time.h>
#include <string.h>
#include "parameters.hpp"

#include <QObject>
#include <QThread>
#include <QVector>

#include "PracticalSocket.h"
#include "receivefrobot.h"

//#define robotPort "64500"
//#define robotIP "10.17.246.210"

struct sendInitInfo
{
    sendInitInfo()
    {
        clean();
    };
    void clean()
    {
        timeStamp = 0;
        statusFlag = 0;
        vx = 0;
        vy = 0;
        vz = 0;
        wx = 0;
        wy = 0;
        wz = 0;
    }

    double timeStamp;
    double statusFlag;
    double vx;
    double vy;
    double vz;
    double wx;
    double wy;
    double wz;
};

class initPos : public QThread
{
public:
    initPos(string aFile);
    void recenter_transmission();           // makes sure data chains are in the right position (ie start at PAD0 = 10)
    void readMatrix();                      // reads a matrix from Port
    int normalizeData(unsigned char s_buf); // normalises the data read from the port
    int normalizeData(unsigned char s_buf,int index);
    void Close();                           // closes communication
    void getMaxFilter(unsigned char* s_buf);// fills in a matrix filter to reduce noise
    virtual void run();                     // main loop
    int getPitchMvt();                      // returns movement to cancel Pitch
    int getRollMvt();                       // returns movement to cancel Roll
    int getYawMvt(int counter, int maxCount);// returns movement to cancel Yaw
    int Send(int index);                    // sends info to robot ("index" determines wether roll, pitch or yaw being controlled : 1=yaw, 2=pitch, 3=roll)
    void SwapData();                        // saves data in oldData
    int listen2socket();                    // receives messages from robot
    int initialExploration();               // exploration to place the reference PAD


    int fd;                                 // reference for file/port link
    int nb_c,nb_l;                          // Matrix dimensions
    string inputfile;                       // serial Port where data is read
    int *matrixFilter;                      // matrix filter filled by 'getMaxFilter'
    QVector< double >  data;                // vector of matrixes where data is stored at time t
    QVector< double > olddata;              // vector of matrixes at time index t-1
    int ASensitivity;                       // sensitivity with which amplitudes are considered equal
    int refIndx;                            // index of the reference PAD
    int pitchIndx;                          // index of PAD used for controlling pitch
    int rollIndx;                           // index of PAD used for controlling roll
    int pdirect;                            // direction of movement for pitch control
    int rdirect;                            // direction of movement for roll control
    double rspeed_ref;                      // rotation speed (reference)
    double rspeed;                          // rotation speed
    double rspeed_step;                     // rotation speed decrease step
    int timeStamp;                          // time stamp for sending messages to robot
    int yawIndx;                            // index of PAD used for controlling yaw
    rspType RSP;                            // message received from robot
};

#endif // INITPOS_H
