#include <boost/interprocess/shared_memory_object.hpp>
#include <boost/interprocess/mapped_region.hpp>
#include <iostream>
#include <cstdlib>
#include <string>
#include <vector>
#include "doc_anonymous_semaphore_shared_data.hpp"
#include <time.h>
#include <boost/date_time/posix_time/posix_time.hpp>

#include "terminal_display.h"
#include "DataClass.h"
#include "DataFILEClass.h"
#include "DataPlayfield.h"
#include "parameters.hpp"


using namespace boost::interprocess;

void writingProc(std::string fileName);
void readingProc(int argc, char * argv[]);
void writingFILEProc(std::string fileName);

int main(int argc, char * argv[])
{
    int f_pid = getpid();                           // father pid
    int s_pid = 0;                                  // son pid
    int pid = 0;                                    // pid of process

    s_pid = fork();                                 // New Process
    if (s_pid < 0)
    {
        std::cout << " Error : could not create new process" << std::endl;
    }
    else
    {
        pid = getpid();
        if (pid == f_pid)                           // Proc : 1
        {
            sleep(1);                               // sleep to give time to other process to create shared memory
            readingProc(argc,argv);
        }
        else                                        // Proc 2 :
        {
            if(useFILE == 0)
            {
                std::string fileName = std::string(SerialPort); // string of reference for serial port
                writingProc(fileName);
            }
            else
            {
                std::string fileName = std::string(FILEpath); // string of reference for data file
                writingFILEProc(fileName);
            }
        }
    }

    return 0;
}

/* reads data from serial port & writes it to shared memory */
void writingProc(std::string fileName)
{
    struct shm_remove                                                       //Remove shared memory on construction and destruction
    {
        shm_remove()
        {
            shared_memory_object::remove("MySharedMemory");
        }
        ~shm_remove()
        {
            shared_memory_object::remove("MySharedMemory");
        }
    } remover;

    shared_memory_object shm(create_only, "MySharedMemory", read_write);    // Create a shared memory object
    shm.truncate(sizeof(shared_memory_buffer));                             // Set size
    mapped_region region(shm , read_write);                                 // Map the whole shared memory in this process
    void * addr = region.get_address();                                     // Get the address of the mapped region
    shared_memory_buffer * data = new (addr) shared_memory_buffer;          // Construct the shared structure in memory

    const int NumMsg = max_nb_matrix_sent;                                  // number of matrixes to be transfered (no limitations on number of matrices read)
    DataClass DATAchain(fileName);                                          // class for reading data from serial port

    int k = 0;                                                              // counter for number of matrixes transfered
    double seconds = acq_period;                                            // time between two acquisitions (in ms) => #define
    boost::posix_time::ptime endwait = boost::posix_time::ptime(boost::posix_time::microsec_clock::universal_time());   // gets current time
    boost::posix_time::time_duration t_elapsed;                                                                         // a duration used for mesuring time between two acquisitions (approx) in order to optimize process utility

    while (k < NumMsg)
    {
        t_elapsed = boost::posix_time::ptime(boost::posix_time::microsec_clock::universal_time()) - endwait;            // time since last acquisition
        if ( t_elapsed.total_milliseconds()>= seconds)									// if elapsed time > acquisition time objective
        {
            endwait = endwait + boost::posix_time::milliseconds(seconds);						// updates time of next acquisition
            DATAchain.readMatrix();											// read data from file
        }

        if (DATAchain.getNbMat()>0  && data->isempty.try_wait())            // writing in shared memory (when possible)
        {
            if (data->mutex.try_wait())
            {
                for (int i=0;i<shared_memory_buffer::NumItemsl;i++)
                {
                    for (int j=0;j<shared_memory_buffer::NumItemsc;j++)
                    {
                        data->items[k % shared_memory_buffer::NumMat][i][j]=DATAchain.getMat(i,j);
                    }
                }
                DATAchain.removeMatrix();
                k++;
                data->mutex.post();
            }
            data->isfull.post();
        }
    }
    DATAchain.Close();
    wait();		// wait for son process to end
}

void writingFILEProc(std::string fileName) {
        //Remove shared memory on construction and destruction
        struct shm_remove {
                shm_remove() {shared_memory_object::remove("MySharedMemory");}
                ~shm_remove() {shared_memory_object::remove("MySharedMemory");}
        } remover;

                        //Create a shared memory object.
        shared_memory_object shm(create_only, "MySharedMemory", read_write);
                        //Set size
        shm.truncate(sizeof(shared_memory_buffer));
                        //Map the whole shared memory in this process
        mapped_region region(shm , read_write);
                        //Get the address of the mapped region
        void * addr = region.get_address();
                        //Construct the shared structure in memory
        shared_memory_buffer * data = new (addr) shared_memory_buffer;

        const int NumMsg =  max_nb_matrix_sent;			// number of matrixes to be transfered (no limitations on number of matrices read)
        int g = getpid();
        DataFILEClass DATAchain(fileName);

        int k = 0;
        double seconds = acq_period; 	// time between two acquisitions (in ms) => #define
        boost::posix_time::ptime endwait = boost::posix_time::ptime(boost::posix_time::microsec_clock::universal_time());
        boost::posix_time::time_duration t_elapsed;


        while (k < NumMsg)
        {
                t_elapsed = boost::posix_time::ptime(boost::posix_time::microsec_clock::universal_time()) - endwait;	//time since last acquisition
                if( t_elapsed.total_milliseconds()>= seconds && DATAchain.checkIndex())									// if elapsed time > acquisition time objective
                {
                        endwait = endwait + boost::posix_time::milliseconds(seconds);							//updates time of next acquisition
//			std::cout<<endwait<<std::endl;
                        DATAchain.readNextMatrix();																// read data from file
//                        std::cout << " : " << DATAchain.getNbMat() << " matrixes in memory"<< std::endl;	//Print >> PIDnumber : Number of matrixes in memory
                }


                // writing in shared memory (when possible)
                if(DATAchain.getNbMat()>0  && data->isempty.try_wait())
                {
                if(data->mutex.try_wait())
                {
                        for(int i=0;i<shared_memory_buffer::NumItemsl;i++)
                        {
                                for(int j=0;j<shared_memory_buffer::NumItemsc;j++)
                                {
                                        data->items[k % shared_memory_buffer::NumMat][i][j]=DATAchain.getMat(i,j);
                                }
                        }
                        DATAchain.removeMatrix();
                        k++;
//                        std::cout << " : matrix n°" << k << " written || " << DATAchain.getNbMat() << " matrixes in memory"<< std::endl;
                        data->mutex.post();
                        }
                        data->isfull.post();
                }
        }

        wait();		// wait for son process to end
//        std::cout << "Writing Proc : The End !" << std::endl;
}

/* Reads data from shared memory & launches simulation */
void readingProc(int argc, char * argv[])
{
    QApplication app(argc, argv);                                               // Qt app for 'real time' display of spikes

    boost::posix_time::ptime startTIME = boost::posix_time::ptime(boost::posix_time::microsec_clock::universal_time());   // gets start time of program
    boost::posix_time::time_duration t_currentRelativeTIME;

    struct shm_remove                                                           // Remove shared memory on destruction
    {
        ~shm_remove()
        {
            shared_memory_object::remove("MySharedMemory");
        }
    } remover;

    shared_memory_object shm(open_only, "MySharedMemory", read_write);          // Create a shared memory object
    mapped_region region(shm, read_write);                                      // Map the whole shared memory in this process
    void * addr = region.get_address();                                         // Get the address of the mapped region
    shared_memory_buffer * data = static_cast<shared_memory_buffer*> (addr);    // Obtain the shared structure

    const int NumMsg = max_nb_matrix_sent;                                                       // Number of matrixes to be read
    double **extracted_data;
    extracted_data = new double*[NBL];
    for (int i = 0;i<NBL;i++)
        extracted_data[i] = new double[NBC];

    std::string tempstr = saveSPIKEfile;
    DataPlayfield * DataOps = new DataPlayfield(NBL,NBC,tempstr);             // class for running model


    //
    //
    //code for writing matrices in file (to be removed once program debugged)
    ofstream savefile;
    savefile.open(saveMATfile);
    savefile << startTIME << "\n";
    //
    //
    //

    int data_counter=0;                   // counter for nb of data packets received
    int matrix_display_counter=6;         // counter for matrix display
    while (data_counter < NumMsg)
    {
        data->isfull.wait();
        data->mutex.wait();
        for (int i=0;i<shared_memory_buffer::NumItemsl;i++)
        {
            for (int j=0;j<shared_memory_buffer::NumItemsc;j++)
            {
                extracted_data[i][j] = data->items[data_counter % shared_memory_buffer::NumMat][i][j];
                /*!!!!!!!!!!!!!!*/
                savefile << extracted_data[i][j]; /*!!!!!!!!!!!!!!*/
                /*!!!!!!!!!!!!!!*/
                savefile << "   "; /*!!!!!!!!!!!!!!*/
            }
        }
        /*!!!!!!!!!!!!!!*/
        t_currentRelativeTIME = boost::posix_time::ptime(boost::posix_time::microsec_clock::universal_time()) - startTIME;
        savefile << t_currentRelativeTIME.total_milliseconds();
        savefile << "\n"; /*!!!!!!!!!!!!!!*/
        savefile.flush(); /*!!!!!!!!!!!!!!*/
        data_counter++;
        data->mutex.post();
        data->isempty.post();
        matrix_display_counter++;
        if(matrix_display_counter >= matrix_display_period)         // display in terminal of acquisition matrix
        {
            display(extracted_data,data_counter);
            matrix_display_counter = 0;
        }
        if (data_counter==1)
            DataOps->firstRun(extracted_data);
        else
            DataOps->play(extracted_data);
    }


    //
    //
    //(to be removed once program debugged)
    savefile.close();
    //
    //
    //

    delete DataOps;
    system(gnuplot_script);
    app.quit();

}
