#include "mainwindow.h"
#include "ui_mainwindow.h"

MainWindow::MainWindow(QWidget *parent)
        : QMainWindow(parent), ui(new Ui::MainWindow)
{
    ui->setupUi(this);

    autotimer_refresh = 200;
    spiketimer_refresh = 200;
    matlabtimer_refresh = 100;//50;//85;
    robottimer_refresh = 50;
//    cerebellumtimer_refresh = 50;
    isRec = false;
    nb_l = 4;
    nb_c = 6;
    isMov = false;
    int size = 20;
    int inter_size = 4;
    int a_size = size + inter_size;
    int xs = 450;                           // horizontal axis for analog matrix display
    int ys = 80;                            // vertical axis for analog matrix display
    timer_refresh = 200;                    // time between two timer events in ms
    //    KColorPatch *akcolorpatch[nb_l*nb_c];
    for(int i=0;i<nb_l;i++)
    {
        for(int j=0;j<nb_c;j++)
        {
            akcolorpatch[i*nb_c+j] = new KColorPatch(ui->centralWidget);
            akcolorpatch[i*nb_c+j]->setObjectName(QString::fromUtf8("kcolorpatch"));
            akcolorpatch[i*nb_c+j]->setGeometry(QRect(xs+a_size*i, ys+a_size*j, size, size));
//                        akcolorpatch[i*nb_c+j]->setColor(QColor(255,255,255,255));
            akcolorpatch[i*nb_c+j]->setColor(QColor(140,140,140,255));
        }
    }

    timer = new QTimer(this);
    AutoStopTimer = new QTimer(this);
    spikeTimer = new QTimer(this);
    connect( timer, SIGNAL(timeout()), this, SLOT(matUpdate()));
    connect( AutoStopTimer, SIGNAL(timeout()), this, SLOT(AutoStopDisplay()));
    //    if(!useFILE)
    //    {
    //        connect( spikeTimer, SIGNAL(timeout()), this, SLOT(spikeDisplay()));
    //    }
    //    else
    //    {
    connect( spikeTimer, SIGNAL(timeout()), this, SLOT(spikeDisplayFile()));
    //    }
    connect(ui->StartStopButton, SIGNAL(clicked()), this, SLOT(startstopRec()));

    matlabTimer = new QTimer(this);
    connect( matlabTimer, SIGNAL(timeout()), this, SLOT(matlabsend()));


    data_vector = new QVector<double>;
    time_vector = new QVector<double>;
    data_vector_edlut = new QVector<double>;
    time_vector_edlut = new QVector<double>;

    ui->qwtPlot->setAxisScale(QwtPlot::yLeft,0,25,0);
    ui->EDLUTqwtPlot->setAxisScale(QwtPlot::yLeft,0,100,0);
    ui->qwtPlot->enableAxis(QwtPlot::xBottom,false);
    ui->EDLUTqwtPlot->enableAxis(QwtPlot::xBottom,false);

    spikeThread=NULL;
    writingThread=NULL;
    writingFileThread=NULL;
    writingSimThread = NULL;

    ui->lineEdit->setReadOnly(true);
    ui->lineEdit->setAlignment(Qt::AlignCenter);
    ui->lineEdit->setText("ND");
    //!!!!!!!!!!!!!!!!!!!!!!
    connect(ui->detButton,SIGNAL(clicked()),this,SLOT(detect()));
    connect(ui->InitButton,SIGNAL(clicked()),this,SLOT(InitPos()));
    recDetect = new recFmatlab();
    recDetect->start();
    //!!!!!!!!!!!!!!!!!!!!!!
    ui->LettersRec->setReadOnly(true);
    ui->LettersRec->setAlignment(Qt::AlignCenter);
    ui->LettersRec->setFontPointSize(19);
    ui->instLetter->setReadOnly(true);
    ui->instLetter->setAlignment(Qt::AlignCenter);
    ui->instLetter->setFontPointSize(55);
    ui->instLetter->setTextColor(QColor("blue"));

    //            QString tmp = QString("\n  A  ");
    //        ui->instLetter->setText(tmp);
    //        ui->instLetter->setTextColor(QColor("blue"));

    robotCMD = new send2robot();
    robotTimer = new QTimer();
    connect(robotTimer,SIGNAL(timeout()),this,SLOT(robotsend()));

    connect(recDetect,SIGNAL(MrspRcTOP(int)),this,SLOT(NewLetter(int)));
    connect(recDetect,SIGNAL(MrspRc(int)),this,SLOT(instLetter(int)));
    connect(recDetect,SIGNAL(updateRobotSpeed(double,double)),this,SLOT(updaterobotspeedX(double,double)));

    rfRobot = new receiveFrobot();
    rfRobot->start();
    connect(rfRobot,SIGNAL(updateRobotFlag(double)),this,SLOT(updaterobotflag(double)));

    connect(ui->manualVButton,SIGNAL(clicked()),this,SLOT(manualTransl()));
    connect(ui->manualWButton,SIGNAL(clicked()),this,SLOT(manualRot()));
    connect(ui->applyButton,SIGNAL(clicked()),this,SLOT(applyOptions()));
    connect(ui->goButton,SIGNAL(clicked()),this,SLOT(manualGO()));
    connect(ui->CancelButton,SIGNAL(clicked()),this,SLOT(CancelOptions()));

    connect(ui->saveBox,SIGNAL(stateChanged(int)),this,SLOT(refreshApplyButton()));
    connect(ui->send2matlabBox,SIGNAL(stateChanged(int)),this,SLOT(refreshApplyButton()));
    connect(ui->send2robotBox,SIGNAL(stateChanged(int)),this,SLOT(refreshApplyButton()));
    connect(ui->positionCorrBox,SIGNAL(stateChanged(int)),this,SLOT(refreshApplyButton()));

    connect(ui->returnButton,SIGNAL(clicked()),this,SLOT(returnToInitPos()));

    ui->saveBox->setCheckState((Qt::CheckState)0);
    ui->send2matlabBox->setCheckState((Qt::CheckState)0);
    ui->send2robotBox->setCheckState((Qt::CheckState)0);
    ui->positionCorrBox->setCheckState((Qt::CheckState)0);

    ui->applyButton->setDisabled(true);

    RD = NULL;

    openBrailleFile();
    connect(rfRobot,SIGNAL(nextLine()),this,SLOT(nextLine()));
    hasClass = 0;
    currPosMarker = 1;

    connect(rfRobot,SIGNAL(posSIG(double)),this,SLOT(checkPos(double)));
    connect(rfRobot,SIGNAL(PauseRec(bool)),this,SLOT(PauseSensorRecording(bool)));

    aCerebellumComm = new CerebellumComm();
    connect(recDetect,SIGNAL(RobotMovforCerebellum(double,double,double,double,double,double,double)),aCerebellumComm,SLOT(add2fifo(double,double,double,double,double,double,double)));
    connect(aCerebellumComm,SIGNAL(updateArmStateInfo(cerebellumCommType)),this,SLOT(GetRobotInfoFromCerebellum(cerebellumCommType)));
    connect(this,SIGNAL(RobotMovforCerebellum(double,double,double,double,double,double,double)),aCerebellumComm,SLOT(add2fifo(double,double,double,double,double,double,double)));

//    connect(this->spikeThread,SIGNAL(OperationCompleteFeedback()),this->aCerebellumComm,SLOT(OperationCompleteFeedbackReceived()));

    connect(recDetect,SIGNAL(updateLetterDetectionResults(double,double,int)),this,SLOT(checkClassResult(double,double,int)));

    if(useFILE==3)
    {
        ui->send2robotBox->setEnabled(false);
    }
    idxI = 0;
    classI = 0;

    ClassResultsFile = fopen(saveClassResults,"w");
    fprintf(ClassResultsFile,"//Classification Result(1=correct|0=false|3=no classification)||Place of letter in text||real Letter index||Letter recognized\n");
}

MainWindow::~MainWindow()
{
    recDetect->terminate();

    if(RD!=NULL)
    {
        RD->close();
        delete RD;
    }

    timer->stop();
    delete timer;
    AutoStopTimer->stop();
    delete AutoStopTimer;
    spikeTimer->stop();
    delete spikeTimer;

    fclose(ClassResultsFile);

    spikeThread->terminate();
    spikeThread->wait();
    delete spikeThread;
    if(useFILE==1)      // using file
    {
        writingFileThread->terminate();
        writingFileThread->wait();
        delete writingFileThread;
    }
    else if(useFILE==2)               // using simulator
    {
        writingSimThread->terminate();
        writingSimThread->wait();
        delete writingSimThread;
    }
    else                    // using port
    {
        writingThread->terminate();
        writingThread->wait();
        delete writingThread;
    }

    data_vector->clear();
    time_vector->clear();
    delete data_vector;
    delete time_vector;
    delete[] akcolorpatch;
    delete ui;
}

void MainWindow::changeColor(int index, int value)
{
    if(value>=0 && value<=200)
    {
        //float tmp = value * 255/200;
        float tmp;
        if(value > 5)
        {
            tmp = value*220/200 + 30 ;
            akcolorpatch[index]->setColor(QColor(0,0,(int)tmp,255));
        }
        else
        {
            tmp = 0;
            //            akcolorpatch[index]->setColor(QColor(255,255,255,0));
            akcolorpatch[index]->setColor(QColor(140,140,140,255));
        }
        //        akcolorpatch[index]->setColor(QColor((int)tmp,0,0,0));    // black

    }
}

void MainWindow::displayMat(double **mat)
{
    for(int i=0;i<nb_l;i++)
    {
        for(int j=0;j<nb_c;j++)
        {
            changeColor(i*nb_c+j,mat[i][j]);
        }
    }
}

void MainWindow::displayMatRaw(double *mat)
{
    for(int i=0;i<24;i++)
    {
        changeColor(i,mat[i]);
    }
}

void MainWindow::matUpdate()
{
    //    displayMat(spikeThread->fingerMat);
    if(useFILE==0)
    {
        ui->lineEdit->setText(QString::number(writingThread->nbMat));
        emit displayPMat(writingThread->DATAchain->rawMat);
        displayMatRaw(writingThread->DATAchain->rawMat);
    }
    else if(useFILE==2)
    {
        ui->lineEdit->setText(QString::number(writingSimThread->nbMat));
        displayMatRaw(writingSimThread->DATAchain->rawMat);
    }
    //    emit displayPMat(spikeThread->fingerMat);
}

void MainWindow::startstopRec()
{
    if(!isRec)
    {
        this->start();
    }
    else
    {
        this->stop();
    }
}

void MainWindow::AutoStopDisplay()
{
    if(!readingThread_isrunning && !writingThread_isrunning)
    {
        this->stop();
    }
}

void MainWindow::start()
{
    int err=0;
    ui->LettersRec->clear();
    ui->instLetter->clear();
    spikeThread = new ProcessThread();

    aCerebellumComm->initSocketTCP();
    connect(this->spikeThread,SIGNAL(OperationCompleteFeedback()),this->aCerebellumComm,SLOT(OperationCompleteFeedbackReceived()));

    init_semaphores();
    //    TimeSaveStruct->reInit();
    //    sharedData = init_sharedData();
    init_sharedDataVars();
    timer->start(timer_refresh);
    AutoStopTimer->start(autotimer_refresh);
    spikeThread->start();
    //!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    matlabTimer->start(matlabtimer_refresh);
    robotTimer->start(robottimer_refresh);
//    cerebellumTimer->start(cerebellumtimer_refresh);
    //!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    if(useSPIKEdisplay==1)
    {
        spikeTimer->start(spiketimer_refresh);
        TimerTimeCounter = 0;
    }
    if(useFILE==1)      // using file
    {
        writingFileThread = new fileThread(s_FILEpath);
        writingFileThread->start();
    }
    else if(useFILE==2) // using simulator
    {
        writingSimThread = new SIMThread();
        writingSimThread->start();
    }
    else if(useFILE==3)     // using cerebellum + simulator
    {
        aCerebellumComm->start();            /////////////////////////////// Cerebellum  communication
        writingSimThread = new SIMThread();
        writingSimThread->start();
    }
    else                // using port
    {
        try
        {
            writingThread = new fingerThread(s_SerialPort);
        }
        catch(int e)
        {
            err = e;
            delete writingThread;
            writingThread = NULL;
            perror("Error initialising fingerThread");
        }
        if(err==0)
        {
            writingThread->start();
        }
    }
    ui->StartStopButton->setText(QApplication::translate("MainWindow", "Stop", 0, QApplication::UnicodeUTF8));
    isRec = true;
    if(err>0)
    {
        this->stop();
    }
}

void MainWindow::stop()
{
    writingThread_isrunning=true;   // to prevent possible blocking of processing thread

    timer->stop();
    //!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    matlabTimer->stop();
    robotTimer->stop();
//    cerebellumTimer->stop();
    //!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    AutoStopTimer->stop();
    if(useSPIKEdisplay==1)
        spikeTimer->stop();
    disconnect(this->spikeThread,SIGNAL(OperationCompleteFeedback()),this->aCerebellumComm,SLOT(OperationCompleteFeedbackReceived()));
    spikeThread->terminate();
    spikeThread->wait();
    delete spikeThread;
    spikeThread = NULL;

    if(useFILE==1)      // using file
    {
        writingFileThread->terminate();
        writingFileThread->wait();
        delete writingFileThread;
        writingFileThread = NULL;
    }
    else if(useFILE==2)     // using simulator
    {
        writingSimThread->terminate();
        writingSimThread->wait();
        delete writingSimThread;
        writingSimThread = NULL;
    }
    else if(useFILE==3)     // using cerebellum + simulator
    {
        aCerebellumComm->terminate();            /////////////////////////////// Cerebellum communication
        aCerebellumComm->clean();
        writingSimThread->terminate();
        writingSimThread->wait();
        delete writingSimThread;
        writingSimThread = NULL;
    }
    else                // using port
    {
        if(writingThread != NULL)
        {
            writingThread->terminate();
            writingThread->wait();
            delete writingThread;
            writingThread = NULL;
        }
        writingThread_isrunning=false;
    }

    if(data_vector->size()!=0)
    {
        data_vector->clear();
        time_vector->clear();
    }

    StoreDisplaySemaphore->acquire(1);
    DisplaySpikeData->clear();
    DisplayTimeData->clear();
    StoreDisplaySemaphore->release(1);
    ui->StartStopButton->setText(QApplication::translate("MainWindow", "Start", 0, QApplication::UnicodeUTF8));
    ui->lineEdit->setText("ND");
    isRec = false;
}

void MainWindow::spikeDisplay()
{
    TimerTimeCounter = TimerTimeCounter + spiketimer_refresh;

    /* display mechanoreceptor spikes */
    StoreDisplaySemaphore->acquire(1);                      // acquiring memory space
    for(int i=0;i<DisplaySpikeData->size();i++)
    {
        data_vector->push_back((double)DisplaySpikeData->at(i));
        time_vector->push_back((double)DisplayTimeData->at(i));
    }
    DisplaySpikeData->clear();
    DisplayTimeData->clear();

    StoreDisplaySemaphore->release(1);                      // releasing memory space

    QwtSymbol * Across = new QwtSymbol();
    Across->setStyle(QwtSymbol::VLine);

    if(!data_vector->isEmpty())
    {
        int timeAXIS = XaxisTimeWindow;
        int tmp = TimerTimeCounter/*time_vector->back()*/ - timeAXIS;
        int i=0;
        while(i<time_vector->size() && time_vector->at(i)<tmp)
        {
            i++;
        }
        if(i>0)
        {
            time_vector->remove(0,i);
            data_vector->remove(0,i);
            if((time_vector->size()>0 && time_vector->front()>tmp) || time_vector->size()==0)
            {
                time_vector->push_front(tmp);
                data_vector->push_front(-1);
            }
        }

        QwtPlotCurve *curve1 = new QwtPlotCurve("SPIKES");
        curve1->setData(time_vector->data(),data_vector->data(),data_vector->size());
        curve1->setStyle(QwtPlotCurve::Dots);

        curve1->setSymbol(*Across);
        ui->qwtPlot->clear();
        curve1->attach(ui->qwtPlot);
        //        ui->qwtPlot->setAxisScale(QwtPlot::xBottom,tmp,time_vector->back(),0);
        ui->qwtPlot->setAxisScale(QwtPlot::xBottom,tmp,/*TimerTimeCounter*/time_vector->back(),timeAXIS/*time_vector->back()-tmp*/);
        ui->qwtPlot->replot();
        delete curve1;
    }

    /* display CN output spikes */
    StoreDisplayEDLUTSemaphore->acquire(1);                      // acquiring memory space
    for(int i=0;i<DisplayEDLUTSpikeData->size();i++)
    {
        data_vector_edlut->push_back((double)DisplayEDLUTSpikeData->at(i));
        time_vector_edlut->push_back((double)DisplayEDLUTTimeData->at(i));
    }
    DisplayEDLUTSpikeData->clear();
    DisplayEDLUTTimeData->clear();

    StoreDisplayEDLUTSemaphore->release(1);                      // releasing memory space

    if(!data_vector_edlut->isEmpty())
    {
        int timeAXIS2 = XaxisTimeWindow;
        int tmp2 = TimerTimeCounter/*time_vector_edlut->back()*/ - timeAXIS2;
        int i2=0;
        while(time_vector_edlut->size()>i2 && time_vector_edlut->at(i2)<tmp2)
        {
            i2++;
        }
        if(i2>0)
        {
            time_vector_edlut->remove(0,i2);
            data_vector_edlut->remove(0,i2);
            if((time_vector_edlut->size()>0 && time_vector_edlut->front()>tmp2) || time_vector_edlut->size() == 0)
            {
                time_vector_edlut->push_front(tmp2);
                data_vector_edlut->push_front(-1);
            }
        }

        QwtPlotCurve *curve2 = new QwtPlotCurve("SPIKES");
        curve2->setData(time_vector_edlut->data(),data_vector_edlut->data(),data_vector_edlut->size());
        curve2->setStyle(QwtPlotCurve::Dots);
        curve2->setSymbol(*Across);
        ui->EDLUTqwtPlot->clear();
        curve2->attach(ui->EDLUTqwtPlot);
        //        ui->EDLUTqwtPlot->setAxisScale(QwtPlot::xBottom,tmp2,time_vector_edlut->back(),0);
        ui->EDLUTqwtPlot->setAxisScale(QwtPlot::xBottom,tmp2,time_vector_edlut->back(),timeAXIS2/*time_vector_edlut->back() - tmp2*/);
        //        QwtScaleDiv NewDiv(tmp2,time_vector_edlut->back());
        //        ui->EDLUTqwtPlot->setAxisScaleDiv(QwtPlot::xBottom,QwtPlot::s
        ui->EDLUTqwtPlot->replot();
        delete curve2;
    }

    delete Across;
}

void MainWindow::spikeDisplayFile()
{
    /* display mechanoreceptor spikes */
    StoreDisplaySemaphore->acquire(1);                      // acquiring memory space
    for(int i=0;i<DisplaySpikeData->size();i++)
    {
        data_vector->push_back((double)DisplaySpikeData->at(i));
        time_vector->push_back((double)DisplayTimeData->at(i));
    }
    DisplaySpikeData->clear();
    DisplayTimeData->clear();

    StoreDisplaySemaphore->release(1);                      // releasing memory space

    QwtSymbol * Across = new QwtSymbol();
    Across->setStyle(QwtSymbol::VLine);

    if(!data_vector->isEmpty())
    {
        int timeAXIS = XaxisTimeWindow;
        int tmp = time_vector->back() - timeAXIS;
        int i=0;
        while(i<time_vector->size() && time_vector->at(i)<tmp)
        {
            i++;
        }
        if(i>0)
        {
            time_vector->remove(0,i);
            data_vector->remove(0,i);
            if((time_vector->size()>0 && time_vector->front()>tmp))
            {
                time_vector->push_front(tmp);
                data_vector->push_front(-1);
            }
        }

        QwtPlotCurve *curve1 = new QwtPlotCurve("SPIKES");
        curve1->setData(time_vector->data(),data_vector->data(),data_vector->size());
        curve1->setStyle(QwtPlotCurve::Dots);

        curve1->setSymbol(*Across);
        ui->qwtPlot->clear();
        curve1->attach(ui->qwtPlot);
        //        ui->qwtPlot->setAxisScale(QwtPlot::xBottom,tmp,time_vector->back(),0);
        ui->qwtPlot->setAxisScale(QwtPlot::xBottom,tmp,time_vector->back(),timeAXIS/*time_vector->back()-tmp*/);
        ui->qwtPlot->replot();
        delete curve1;
    }

    /* display CN output spikes */
    StoreDisplayEDLUTSemaphore->acquire(1);                      // acquiring memory space
    for(int i=0;i<DisplayEDLUTSpikeData->size();i++)
    {
        data_vector_edlut->push_back((double)DisplayEDLUTSpikeData->at(i));
        time_vector_edlut->push_back((double)DisplayEDLUTTimeData->at(i));
    }
    DisplayEDLUTSpikeData->clear();
    DisplayEDLUTTimeData->clear();

    StoreDisplayEDLUTSemaphore->release(1);                      // releasing memory space

    if(!data_vector_edlut->isEmpty())
    {
        int timeAXIS2 = XaxisTimeWindow;
        int tmp2 = time_vector_edlut->back() - timeAXIS2;
        int i2=0;
        while(time_vector_edlut->size()>i2 && time_vector_edlut->at(i2)<tmp2)
        {
            i2++;
        }
        if(i2>0)
        {
            time_vector_edlut->remove(0,i2);
            data_vector_edlut->remove(0,i2);
            if((time_vector_edlut->size()>0 && time_vector_edlut->front()>tmp2) || time_vector_edlut->size() == 0)
            {
                time_vector_edlut->push_front(tmp2);
                data_vector_edlut->push_front(-1);
            }
        }

        QwtPlotCurve *curve2 = new QwtPlotCurve("SPIKES");
        curve2->setData(time_vector_edlut->data(),data_vector_edlut->data(),data_vector_edlut->size());
        curve2->setStyle(QwtPlotCurve::Dots);
        curve2->setSymbol(*Across);
        ui->EDLUTqwtPlot->clear();
        curve2->attach(ui->EDLUTqwtPlot);
        //        ui->EDLUTqwtPlot->setAxisScale(QwtPlot::xBottom,tmp2,time_vector_edlut->back(),0);
        ui->EDLUTqwtPlot->setAxisScale(QwtPlot::xBottom,tmp2,time_vector_edlut->back(),timeAXIS2/*time_vector_edlut->back() - tmp2*/);
        ui->EDLUTqwtPlot->replot();
        delete curve2;
    }

    delete Across;
}

//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
void MainWindow::matlabsend()
{
    edlutSemaphore->acquire(1);
    int tmp = eSim->Sproc->letter_index;
    if (tmp >= letter_count-1)
    {
        tmp = -1;//0;
        eSim->Sproc->letter_index=tmp;
//        nextLine();
    }
    letter_index = tmp;
//    sharedRSPSemaphore->acquire(1);     // maybe replace acquire by SIGNAL+SLOT
//    eSim->MATLAB->data.statusFlag = sharedRSP->Rstatus;
//    sharedRSPSemaphore->release(1);
//    eSim->MATLAB->data.LetterFlag = tmp;
    eSim->MATLAB->data.timeStamp++;
    eSim->MATLAB->Send();
    edlutSemaphore->release(1);
}

void MainWindow::detect()
{
    edlutSemaphore->acquire(1);
    eSim->Sproc->classification();
    edlutSemaphore->release(1);
}

void MainWindow::InitPos()
{
    //    initPos *aInit = new initPos(s_SerialPort);
    //    aInit->start();
    //    aInit->wait();
    //
    //    delete aInit;
    edlutSemaphore->acquire(1);
    eSim->Sproc->setInitPos();
    edlutSemaphore->release(1);

}


void MainWindow::NewLetter(int clas)
{
    if(clas>0)
    {

        char symb[30] = "ABCDEFGHIJKLMNOPQRSTUVWXYZ  -";
        int TMP = detectNextLetter();
        hasClass = 1;

        if (clas == TMP)
        {
            ui->instLetter->setTextColor(QColor("green"));
            QString tmp = QString(ui->LettersRec->toPlainText());
            tmp.append(symb[TMP-1]);
            tmp.append(" ");
            ui->LettersRec->setText(tmp);
        }
        else
        {
            ui->instLetter->setTextColor(QColor("red"));
            QString tmp = QString(ui->LettersRec->toPlainText());
            tmp.append("-");
            tmp.append(" ");
            ui->LettersRec->setText(tmp);
        }

        //        ui->instLetter->setTextColor(QColor("green"));
        //        //char alpha[29] = "ABCDEFGHIJKLMNOPQRSTUVWXYZ \n";
        //        char symb[30] = "ABCDEFGHIJKLMNOPQRSTUVWXYZ  -";
        //        //        QString tmp = QString(ui->LettersRec->toPlainText());
        //        //tmp.append(alpha[clas-1]);
        //        int tmpINT = clas;//checkLetterDisc(clas);
        //        if(tmpINT == -1)
        //        {
        //            //            tmp.append(QString("\n"));
        //            ui->LettersRec->append(QString("\n"));
        //        }
        //        else
        //        {
        //            QString tmp = QString(ui->LettersRec->toPlainText());
        //            tmp.append(symb[tmpINT-1]);
        //            tmp.append(" ");
        //            ui->LettersRec->setText(tmp);
        //        }
        //        //        tmp.append(" ");
        //        //        ui->LettersRec->setText(tmp);

        QTextCursor c =  ui->LettersRec->textCursor();
        c.movePosition(QTextCursor::End);
        ui->LettersRec->setTextCursor(c);
    }
}

int MainWindow::checkLetterDisc(int clas)
{
    //    int tmp = WordIdx.first();
    //    WordIdx.pop_front();
    //    WordIdx.push_back(tmp);
    int tmp = WordIdx[letter_index];

    if(tmp == clas)
    {
        return clas;
    }
    else if(clas == 28)
    {
        return -1;
    }
    else
    {
        return 29;
    }
}

void MainWindow::instLetter(int clas)
{
//    printf("RECEIVED LETTER 2 : %d  - - - -\n",clas-1);
    if(clas>=1)
    {
        char alpha[30] = "ABCDEFGHIJKLMNOPQRSTUVWXYZ   ";
        QString tmp = QString("\n  ");
        tmp.append(alpha[clas-1]);
        ui->instLetter->setText(tmp);
        ui->instLetter->setTextColor(QColor("blue"));
    }
}

void MainWindow::updaterobotspeedX(double vx,double vy)
{
//    printf("vx : int %d - double %f || vy : int %d - double %f\n",vx,vx,vy,vy);
    robotCMD->rCMD.vx = vx;
    robotCMD->rCMD.vy = vy;
}

void MainWindow::robotsend()
{
    useControlOKSemaphore->acquire(1);
    bool tmp = useControlOK;
    useControlOKSemaphore->release(1);
    if(tmp)
    {
        edlutSemaphore->acquire(1);
        robotCMD->rCMD.vy = eSim->Sproc->Vy;
        edlutSemaphore->release(1);
    }
    else
    {
        robotCMD->rCMD.vy = 0;//-0.005;
    }
    robotCMD->rCMD.vz = 0;//-0.003;

    if(robotCMD->rCMD.statusflag == 3)
    {
        if(robotSPEED>0)                        // if robotspeed is >0 in config file then consider it, else consider speed sent by JOKER decision process
        {
            robotCMD->rCMD.vx = (double)robotSPEED;
        }
    }
    else
    {
        robotCMD->rCMD.vx = 0;
    }
    //        printf("Flag = %f,  Vx = %f,  Vy = %f,  Vz = %f\n",robotCMD->rCMD.statusflag,robotCMD->rCMD.vx,robotCMD->rCMD.vy,robotCMD->rCMD.vz);
    robotCMD->sendinfo();
    if(useFILE==3)
    {
        if(robotSPEED>0)
            emit RobotMovforCerebellum((double)0,(double)0,((double)robotSPEED)/10,(double)0,(double)0,(double)0,(double)1);
        else
            emit RobotMovforCerebellum((double)0,(double)0,((double)robotCMD->rCMD.vx)/10,((double)robotCMD->rCMD.vy)/10,(double)0,(double)0,(double)1);
    }
}

void MainWindow::updaterobotflag(double flag)
{
    if(flag !=3 || (flag == 3 && isMov))
    {
        robotCMD->rCMD.statusflag = flag;
    }
}

void MainWindow::nextLine()
{
    //        NewLetter(28);
    ui->LettersRec->append(QString("\n"));
    QTextCursor c =  ui->LettersRec->textCursor();
    c.movePosition(QTextCursor::End);
    ui->LettersRec->setTextCursor(c);
    letter_index = -1;

    hasClass = 0;
    currPosMarker = 1;
}

void MainWindow::manualRot()
{
    if(robotCMD->rCMD.statusflag!=1)
    {
        robotCMD->rCMD.clean();
        robotCMD->rCMD.statusflag = 1;
        ui->manualWButton->setText("STOP Rot.");
        ui->goButton->setText("Go !");
        ui->manualVButton->setText("Translation");

        if(RD == NULL)
        {
            RD = new RawDisplay();
            Qt::WindowFlags flag=Qt::WindowStaysOnTopHint;
            RD->setWindowFlags(flag);
            RD->show();
            connect(this,SIGNAL(displayPMat(double*)),RD,SLOT(DisplayMat(double*)));

        }
        //        else
        //        {
        //            RD->setFocus();
        //        }
    }
    else
    {
        robotCMD->rCMD.clean();
        robotCMD->rCMD.statusflag = 0;
        ui->manualWButton->setText("Rotation");

        if(RD != NULL)
        {
            disconnect(this,SIGNAL(displayPMat(double*)),RD,SLOT(DisplayMat(double*)));
            RD->close();
            delete RD;
            RD = NULL;
        }
    }
}

void MainWindow::manualTransl()
{
    if(robotCMD->rCMD.statusflag!=2)
    {
        robotCMD->rCMD.clean();
        robotCMD->rCMD.statusflag = 2;
        ui->manualVButton->setText("STOP Trans.");
        ui->goButton->setText("Go !");
        ui->manualWButton->setText("Rotation");

        if(RD==NULL)
        {
            RD = new RawDisplay();
            Qt::WindowFlags flag=Qt::WindowStaysOnTopHint;
            RD->setWindowFlags(flag);
            RD->show();
            connect(this,SIGNAL(displayPMat(double*)),RD,SLOT(DisplayMat(double*)));
        }
    }
    else
    {
        robotCMD->rCMD.clean();
        robotCMD->rCMD.statusflag = 0;
        ui->manualVButton->setText("Translation");

        if(RD!=NULL)
        {
            disconnect(this,SIGNAL(displayPMat(double**)),RD,SLOT(DisplayMat(double**)));
            RD->close();
            delete RD;
            RD = NULL;
        }
    }
}

void MainWindow::manualGO()
{
//    if(robotCMD->rCMD.statusflag!=3)
    printf("Robot CMD: %f\n",robotCMD->rCMD.statusflag);
    if(!isMov)
    {
//        if(robotCMD->rCMD.statusflag>=0)
//        {
//            robotCMD->rCMD.clean();
//            robotCMD->rCMD.statusflag = 3;
//        }
        if(rfRobot->RSP.Rstatus>=0)
        {
            robotCMD->rCMD.clean();
            robotCMD->rCMD.statusflag = 3;
        }
        isMov = true;
        ui->goButton->setText("STOP Mov.");
        ui->manualWButton->setText("Rotation");
        ui->manualVButton->setText("Translation");

        if(RD!=NULL)
        {
            disconnect(this,SIGNAL(displayPMat(double**)),RD,SLOT(DisplayMat(double**)));
            RD->close();
            delete RD;
            RD = NULL;
        }
    }
    else
    {
        isMov = false;
        if(robotCMD->rCMD.statusflag==3)
        {
            robotCMD->rCMD.clean();
            robotCMD->rCMD.statusflag = 0;
        }
        ui->goButton->setText("Go !");
    }
}


void MainWindow::applyOptions()
{
    if(ui->saveBox->isChecked())
    {
        saveDataOKSemaphore->acquire(1);
        saveDataOK = true;
        saveDataOKSemaphore->release(1);
        edlutSemaphore->acquire(1);
        eSim->saveData = true;
        eSim->Sproc->saveData = true;
        edlutSemaphore->release(1);
    }
    else
    {
        saveDataOKSemaphore->acquire(1);
        saveDataOK = false;
        saveDataOKSemaphore->release(1);
        edlutSemaphore->acquire(1);
        eSim->saveData = false;
        eSim->Sproc->saveData = false;
        edlutSemaphore->release(1);
    }
    if(ui->send2matlabBox->isChecked())
    {
        send2MatlabOKSemaphore->acquire(1);
        send2MatlabOK = true;
        send2MatlabOKSemaphore->release(1);
    }
    else
    {
        send2MatlabOKSemaphore->acquire(1);
        send2MatlabOK = false;
        send2MatlabOKSemaphore->release(1);
    }
    if(ui->send2robotBox->isChecked())
    {
        send2RobotOKSemaphore->acquire(1);
        send2RobotOK = true;
        send2RobotOKSemaphore->release(1);
    }
    else
    {
        send2RobotOKSemaphore->acquire(1);
        send2RobotOK = false;
        send2RobotOKSemaphore->release(1);
    }
    if(ui->positionCorrBox->isChecked())
    {
        useControlOKSemaphore->acquire(1);
        useControlOK = true;
        useControlOKSemaphore->release(1);
    }
    else
    {
        useControlOKSemaphore->acquire(1);
        useControlOK = false;
        useControlOKSemaphore->release(1);
    }
    ui->applyButton->setDisabled(true);
}

void MainWindow::CancelOptions()
{
    send2RobotOKSemaphore->acquire(1);
    int rbtSend = 2*(int)send2RobotOK;
    send2RobotOKSemaphore->release(1);

    send2MatlabOKSemaphore->acquire(1);
    int mtSend = 2*(int)send2MatlabOK;
    send2MatlabOKSemaphore->release(1);

    saveDataOKSemaphore->acquire(1);
    int sav = 2*(int)saveDataOK;
    saveDataOKSemaphore->release(1);

    useControlOKSemaphore->acquire(1);
    int corr = 2*(int)useControlOK;
    useControlOKSemaphore->release(1);

    ui->saveBox->setCheckState((Qt::CheckState)sav);
    ui->send2matlabBox->setCheckState((Qt::CheckState)mtSend);
    ui->send2robotBox->setCheckState((Qt::CheckState)rbtSend);
    ui->positionCorrBox->setCheckState((Qt::CheckState)corr);

    ui->applyButton->setDisabled(true);
}

void MainWindow::refreshApplyButton()
{
    ui->applyButton->setDisabled(false);
}


void MainWindow::openBrailleFile()
{
    ifstream File(BrailleFile);
    if (!File.is_open()) // if unable to open file
    {
        std::cout << "Error opening data file !" << std::endl;
        return;
    }

    QString Word;
    char tmpSTR[2];
    int tmpINT;
    double tmpDx;
    double tmpDy;
    while(!File.eof())
    {
        File >> tmpSTR;
        File >> tmpINT;
        File >> tmpDx;
        File >> tmpDy;
        if(!File.eof())
        {
            Word.append(tmpSTR);
            Word.append(" ");
            WordIdx.push_back(tmpINT);
            WordPosX.push_back(tmpDx);
            WordPosY.push_back(tmpDy);
        }
    }

    File.close();

    ui->BrailleLine->setReadOnly(true);
    ui->BrailleLine->setAlignment(Qt::AlignCenter);
    ui->BrailleLine->setFontPointSize(18);
    ui->BrailleLine->setText(Word);
    letter_index = -1;
    letter_count = WordIdx.size();

//    for(int i=0;i<WordPosX.size();i++)
//    {
//        WordPosX[i] = WordPosX[i] + RInitPosX;               //!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! deal with mm/m/cm ...
//        WordPosY[i] = WordPosY[i] + RInitPosY;
//    }
}

int MainWindow::detectNextLetter()
{
    double currPos;
    double FLAG;
    sharedRSPSemaphore->acquire(1);     // maybe replace acquire by SIGNAL+SLOT
    currPos = sharedRSP->xpos;
    FLAG = sharedRSP->Rstatus;
    sharedRSPSemaphore->release(1);

    if(/*FLAG == 1 &&*/ currPos<WordPosX.last())        // in DLR version, should be : currPos>WordPosX.last() (because arm moves along Xnegative)
    {
        int i=0;
        while(i<WordPosX.size() && currPos > WordPosX[i])
        {
            i++;
        }
        return WordIdx[i];
    }
    return 28;
}

void MainWindow::checkPos(double currPos)
{
    if(currPosMarker<WordIdx.size() && currPos > WordPosX[currPosMarker] && hasClass==0) // in DLR version, should be : currPos<WordPosX[currPosMarker] (because arm moves along Xnegative)
    {
        NewLetter(28);
    }
    hasClass = 0;
    currPosMarker ++;
    if(currPosMarker>WordPosX.size())
    {
        currPosMarker=1;
    }
}

void MainWindow::PauseSensorRecording(bool aRec)
{
    doRecordSemaphore->acquire(1);
    doRecord = aRec;
    doRecordSemaphore->release(1);
}

void MainWindow::returnToInitPos()
{
    if(rfRobot->RSP.Rstatus!=-3)
    {
        isMov = false;
        robotCMD->rCMD.clean();
        robotCMD->rCMD.statusflag = -1;
        ui->goButton->setText("Go !");
    }

}

void MainWindow::GetRobotInfoFromCerebellum(cerebellumCommType tmp)
{
    // Storing Robot state to be sent to Matlab Joker
    sharedRSPSemaphore->acquire(1);     // maybe replace acquire by SIGNAL+SLOT
    sharedRSP->clean();
    sharedRSP->xspeed = tmp.vX;
    sharedRSP->yspeed = tmp.vY;
    sharedRSP->timestamp = tmp.time;
    sharedRSP->Rstatus = tmp.statusflag;
    sharedRSP->xpos = tmp.posX;
    sharedRSP->ypos = tmp.posY;
    sharedRSPSemaphore->release(1);

    cout << "Simulator Command update Stored : posX = " << sharedRSP->xpos << " || posY = "  << sharedRSP->ypos << endl;

    // Saving trajectory info in file (to be implemented if necessary)
    checkClassResult(tmp.posX*10, tmp.posY*10, 27);

}

void MainWindow::checkClassResult(double posX, double posY, int classs)
{
//    FILE * fd;
//    fd = fopen(saveClassResults,"a+");
    int i = 0;
    double Wf = (double)fingerWidth/100;
    double Hf = (double)fingerHeight/100;
    double Wl = (double)letterWidth/100;
    double Hl = (double)letterHeight/100;
    double fSp = fingerSprecision;
    while(i<WordIdx.size() && !(posX+Wf+fSp>=WordPosX[i] && posX-fSp<=WordPosX[i]+Wl && posY+fSp>=WordPosY[i] && posY-fSp<=WordPosY[i]))
    {
        i++;
    }
    if(i>=WordIdx.size())
    {
        return;
    }
    else if(!classI && i!=idxI)              // the finger has passed to a new letter (and no classification has been made)
    {
        fprintf(ClassResultsFile,"3 %d %d -1\n",idxI,WordIdx[idxI]);
        classI = 0;
        idxI = i;
    }
    else if(classI && i!=idxI)              // the finger has passed to a new letter (and a classification has already been made)
    {
        classI = 0;
        idxI = i;
    }
    else
    {
        if(classs!=27)    // a classification has been made
        {
            if(classs==WordIdx[idxI])   // classification correct
            {
                fprintf(ClassResultsFile,"1 %d %d %d\n",idxI,WordIdx[idxI],classs);
            }
            else                        // wrong classification
            {
                fprintf(ClassResultsFile,"0 %d %d %d\n",idxI,WordIdx[idxI],classs);
            }
            classI = 1;
            idxI = i;
        }
    }

    fflush(ClassResultsFile);
}
