#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QtGui/QMainWindow>
#include <QTimer>
#include <iostream>
#include "kcolordialog.h"
#include "processthread.h"
#include "fingerthread.h"
#include "globalVar.h"
#include "filethread.h"
#include "edlut.h"
#include <qwt_plot_curve.h>
#include <QVector>
#include <QString>
#include "recfmatlab.h"
#include "send2robot.h"
#include "receivefrobot.h"
#include "simthread.h"

#include <qwt_symbol.h>
#include <qwt_scale_div.h>

#include <iostream>
#include <fstream>

//#include "initpos.h"
#include "rawdisplay.h"
#include "parameters.hpp"
#include "cerebellumcomm.h"

//#define RobotSpeedDefined 20

#define RInitPosY 0
#define RInitPosX -0.135

namespace Ui
{
    class MainWindow;
}

class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    MainWindow(QWidget *parent = 0);
    ~MainWindow();

    int nb_c;
    int nb_l;
    bool isRec;      // is recording data
    bool isMov;      // describes if robot is moving forward (ie, if can follow commands)


    KColorPatch *akcolorpatch[24];

    void changeColor(int index, int value);     // changes the color of a patch
    void displayMat(double **mat);                 // updates colors of all patchs
    void displayMatRaw(double *mat);                // updates colors of all patchs with raw data values
    void openBrailleFile();
    int checkLetterDisc(int clas);                         // checks if letter is correctly classified

    QTimer * timer;
    QTimer * AutoStopTimer;
    QTimer * spikeTimer;
    ProcessThread *spikeThread;                     // thread generating spikes
    fingerThread *writingThread;
    fileThread *writingFileThread;
    SIMThread *writingSimThread;
    int timer_refresh;
    int autotimer_refresh;
    int spiketimer_refresh;
    int matlabtimer_refresh;
    QVector<double> * data_vector;
    QVector<double> * time_vector;
    QVector<double> * data_vector_edlut;
    QVector<double> * time_vector_edlut;
    void start();
    void stop();
    QVector <int> WordIdx;                  // vector containing indexes of letters in Braille Line
    QVector <double> WordPosX;
    QVector <double> WordPosY;
    int letter_index;
    int letter_count;

    QTimer * matlabTimer;
    recFmatlab * recDetect;
    QTimer * robotTimer;
    int robottimer_refresh;
    send2robot * robotCMD;
    receiveFrobot * rfRobot;
    QTimer * cerebellumTimer;
    CerebellumComm * aCerebellumComm;
//    int cerebellumtimer_refresh;

    RawDisplay * RD;
    double TimerTimeCounter;

    int detectNextLetter();
//    int NextLetter;
//    int detectionFLAG;
    int hasClass;
    int currPosMarker;

    int idxI;           // index of letter currently being read (found using positions)
    bool classI;        // 0 if classification has not yet been made
    FILE * ClassResultsFile;        // stream in which classification results will be saved


public slots:
    void matUpdate();
    void startstopRec();
    void AutoStopDisplay();
    void spikeDisplay();
    void spikeDisplayFile();
    void detect();
    void InitPos();
    void NewLetter(int clas);
    void instLetter(int clas);
    void robotsend();
    void updaterobotspeedX(double vx,double vy);
    void updaterobotflag(double flag);

    void matlabsend();
    void manualRot();
    void manualTransl();
    void applyOptions();
    void manualGO();
    void CancelOptions();
    void refreshApplyButton();
    void nextLine();
    void checkPos(double currPos);
    void PauseSensorRecording(bool aRec);
    void returnToInitPos();
    void GetRobotInfoFromCerebellum(cerebellumCommType tmp);    // updates robot tate info according to what is received from cerebellum
    void checkClassResult(double posX,double posY, int classs);             // checks the position of the finger and consequently the actual stimuli being read !

signals:
    void displayPMat(double*);
    void RobotMovforCerebellum(double,double,double,double,double,double,double);    //signal for sending x,y,z speeds and accelerations to cerebellum

private:
    Ui::MainWindow *ui;
};

#endif // MAINWINDOW_H
