//( parameters.hpp

#ifndef PARAMETERS_HPP
#define PARAMETERS_HPP

#include <iostream>
#include <fstream>
#include <iostream>
#include <string>
#include <boost/algorithm/string/split.hpp>
#include <vector>
#include <sstream>

#define configfile "../conf/spikebuilder.conf"

/*******************************************************
 *
 * JP parameters
 *
 *******************************************************/
using namespace std;

float intParser(const char* achar);                                     // parses file for a specific name and returns corresponding float value
string charParser(const char* achar);                                   // parses file for a specific name and returns corresponding string value
float intMissingPARAM(const char* achar);                               // assigns default float value whan name not found in file
string charMissingPARAM(const char* achar);                             // assigns default string value whan name not found in file


extern const string s_SerialPort;                       // name of the serial port
extern const char* SerialPort;
extern const int nb_char;                               // number of 'chars' to be read per packet
extern const string s_sensibility;                      // sensibility of mecanoreceptors (C = low | D = medium | E = high) /* NOT USED */
extern const char* sensibility;
extern const int PAD_sleep_state;                       // value of PAD during sleeping state
extern const int NBL;                                   // number of lines in finger matrix
extern const int NBC;                                   // number of colomns in finger matrix
extern const int acq_period;                            // time between two acquisitions in ms (lower bound)

extern const int matrix_display_period;                 // period between two displays of finger matrix (in nb of data aquisitions)
extern const int spike_display_period;                  // period between two refresh of spike plot (in nb of data aquisitions)
extern const int nbSpikePoints;                         // number of time "points" in spike plot
extern const int XaxisTimeWindow;                       // length of the xAxis for spike plot (in ms)

extern const int max_nb_matrix_sent;                    // number of matrixes to be sent (TEMPORARY : as ong as no manual stop option implemented !!!)

extern const string s_saveSPIKEfile;                    // file to save spikes in
extern const char* saveSPIKEfile;
extern const string s_saveMATfile;                      // file to save filtered finger acquisition matrixes in
extern const char* saveMATfile;

extern const float gaussian_radius;                     // gaussian kernel first radius
extern const float gaussian_radius2;                    // gaussian kernel second radius
extern const float synapse_sensibility;                 // multiplier factor of mecanoreceptor data
extern const float static_threshold;                    // threshold to limit finger noise (in % of sleep state value)
extern const float dynamic_threshold;                   // threshold to limit finger noise (in % of maximum currently mesured value)

extern const int useFILE;                               // 1 if read from file || 0 read from serial port
extern const string s_FILEpath;                         // path of the file were data will be read
extern const char* FILEpath;
extern const int useFILEfilter;                         // 1 use filter on data read from file || 0 ignore filter when reading from data file

extern const int  useSPIKEdisplay;                      // 1 displays spikes in 'real time' || 0 does not display spikes in real time

extern const int readSpeed;                             // 1 reads speed from mat file || 0 no speed in mat file

extern const string s_saveUnMATfile;                    // file to save unfiltered finger acquisition matrixes in
extern const char* saveUnMATfile;
extern const string s_saveCNfile;                       // file to save cuneate nucleus spikes in
extern const char* saveCNfile;

//extern const string s_CN_WGH;                           // file containing EDLUT CN weights
//extern const char* CN_WGH;
//extern const string s_CN_NET;                           // file containing EDLUT CN network description
//extern const char* CN_NET;
extern const char * CN_WGH;                             // config file reference to file containing EDLUT CN weights
extern const char * CN_NET;                             // config file reference to file containing EDLUT CN network description
extern const int EDLUTtimeDelay;                        // delay of upward communication of sensoritmotor information (implimented at EDLUT level) in ms

extern const int saveCNspikes;                          // 1 save cuneate nucleus spikes in file || 0 do not save cuneate nucleus spikes in file
extern const int saveLIFspikes;                         // 1 save mechanoreceptor spikes in file || 0 do not save mechanoreceptor spikes in file
extern const int saveFILTEREDmat;                       // 1 save filtered matrices in file || 0 do not save filtered matrices in file
extern const int saveRAWmat;                            // 1 save raw matrices in file || 0 do not save raw matrices in file
extern const int useCN;                                 // 1 use EDLUT cuneate nucleus model || 0 do not use cuneate nucleus

extern const char * MatlabAddress;                      // LAN address of matlab classification process (for UDP connection)
extern const char * ThisAddressM;                       // LAN address of the computer running SpikeBuilder (viewed by Matlab)
extern const char * SendPort;                           // Port used to send data to matlab classification process
extern const char * RecPort;                            // Port for receiving results of classification from Matlab

extern const char * robotAddress;                       // LAN address of the robot controller
extern const char * ThisAddressR;			// LAN address of the computer running SpikeBuilder (viewed by Robot)
extern const char * SendRobotPort;                      // Port used to send orders to robot
extern const char * RecRobotPort;                       // Port used for receiving info from robot

extern const char * SimulatorAddress;                   // LAN address of the finger Simulator process
extern const char * ThisAddressS;			// LAN address of the computer running SpikeBuilder (viewed by Simulator)
extern const char * SendFingerSIMPort;                  // Port used to send orders to Simulator
extern const char * RecFingerSIMPort;                   // Port used for receiving finger sensor data from Simulator

extern const int seperation_Threshold;                  // threshold for detecting a new letter on the sensors

//extern const int CNsize;                               // size of the CN network (ie number of output neurons)

extern const char * BrailleFile;                        // file containing Braille Line to be read

extern const int robotSPEED;                            // Mvt Speed of robot

extern const char * CerebellumAddress;                  // LAN address of computer running Cerebellum
extern const char * SendCerebellumPort;                 // Port used to send information to cerebellum
extern const int CerebellumCommDelay;			// Delay in communication between Joker and Cerebellum via SpikeBuilder (in ms)

extern const int fingerWidth;                           // width of the finger (in 10^-2mm)
extern const int fingerHeight;                          // height of the sensitive part of the finger (in 10^-2mm)
extern const int fingerSprecision;			// precision of finger - in mm
extern const int letterWidth;				// width of a Braille Letter (ine 10^-2mm)
extern const int letterHeight;				// height of a Braille Letter (ine 10^-2mm)

extern const char * saveClassResults;                      // file to save classification results

/*******************************************************
 *
 * General parameters
 *
 *******************************************************/
/**
 *
 * Number of pyramidal cell in the attractive network
 *
 **/
const int numberOfPyramidalCell = 50;

/**
 *
 * Intensity to input to a neuron for testing purpose only
 *
 **/
const double inputTestingIntensity = -500.5;

/**
 *
 * Weight of a stimulus neuron
 *
 **/
const double stimulusNeuronWeight = 2400;

/**
 *
 * Intensity of the stimulus
 *
 **/
const double stimulusIntensity = 10;//0.1;

/**
 *
 * Intensity of the noise inside the network
 *
 **/
const double internNoiseIntensity = 0.05;

/**
 *
 * Time step used during the simulation
 *
 **/
const double timeStep = 0.001;

/**
 *
 * Minimum weight necessary to create a synapse
 *
 **/
const double minimumSynapseWeight = 0;

/*******************************************************
 *
 * Parameters for myLifNeuron
 *
 *******************************************************/


/**
 *
 * Reset potential (mV)
 *
 **/
const double myLifNeuronResetPotential = -100;//-75

/**
 *
 * Resting potential (mV)
 *
 **/
const double myLifNeuronRestingPotential = -70;

/**
 *
 * Firing threshold potential (mV)
 *
 **/
const double myLifNeuronFiringThreshold = -50;

/**
 *
 * Membrane capacitance (nFarad)
 *
 **/
const double myLifNeuronMembraneCapacitance = 0.5;

/**
 *
 * Membrane leak conductance (nSiemens)
 *
 **/
const double myLifNeuronLeakConductance = 25;

/**
 *
 * Refactory period (Second)
 *
 **/
const double myLifNeuronRefractoryPeriod = 0.002;

/**
*
*   Parameters for threshold fatigue
*
**/

const double myLifNeuronRestingFiringThreshold = -50;

const double tau_threshold =0.100;

const double A_threshold = 50;


/*******************************************************
 *
 * Parameters for Pyramidal cells
 *
 *******************************************************/


/**
 *
 * Reset potential of a pyramidal cell (mV)
 *
 **/
const double pyramidalCellResetPotential = -100; //-55//-75;

/**
 *
 * Resting potential of a pyramidal cell (mV)
 *
 **/
const double pyramidalCellRestingPotential = -70;

/**
 *
 * Firing threshold potential of a pyramidal cell (mV)
 *
 **/
const double pyramidalCellFiringThreshold = -50;

/**
 *
 * Membrane capacitance of a pyramidal cell (Farad)
 *
 **/
const double pyramidalCellMembraneCapacitance = 0.5; //.5 ca donne 20ms de tps carac,

/**
 *
 * Membrane leak conductance of a pyramidal cell (Siemens)
 *
 **/
const double pyramidalCellLeakConductance = 25;

/**
 *
 * Refactory period of a pyramidal cell (Second)
 *
 **/
const double pyramidalCellRefractoryPeriod = 0.002;//0.002;




/*******************************************************
 *
 * Parameters for Interneuron cells
 *
 *******************************************************/


/**
 *
 * Reset potential of an interneuron (mV)
 *
 **/
const double interneuronResetPotential = -55;  //-75;

/**
 *
 * Resting potential of an interneuron (mV)
 *
 **/
const double interneuronRestingPotential = -70;

/**
 *
 * Firing threshold potential of an interneuron (mV)
 *
 **/
const double interneuronFiringThreshold = -50;

/**
 *
 * Membrane capacitance of an interneuron (Farad)
 *
 **/
const double interneuronMembraneCapacitance = 0.2;

/**
 *
 * Membrane leak conductance of an interneuron (Siemens)
 *
 **/
const double interneuronLeakConductance = 20;

/**
 *
 * Refactory period of an interneuron (Second)
 *
 **/
const double interneuronRefractoryPeriod = 0.001;


/*******************************************************
 *
 * Parameters for synapse using AMPA receptors
 *
 *******************************************************/

/**
 *
 * Decay time of AMPA receptors (Second)
 *
 **/
const double ampaDecayTime = 0.002;

/**
 *
 * Reference potential of AMPA receptor (mV)
 *
 **/
const double ampaReferencePotential = 0;

/**
 *
 * Leak conductance for recurrent AMPA receptor with pyramidal cell
 *
 **/
const double ampaRecurrentPyramidalCellLeakConductance = 0.104; //0.5

/**
 *
 * Leak conductance for extern AMPA receptor with pyramidal cell
 *
 **/
const double ampaExternPyramidalCellLeakConductance = 2.08; //2.08

/**
 *
 * Leak conductance for recurrent AMPA receptor with interneurons
 *
 **/
const double ampaRecurrentInterneuronsLeakConductance = 0.081; //0.4

/**
 *
 * Leak conductance for extern AMPA receptor with interneurons
 *
 **/
const double ampaExternInterneuronsLeakConductance = 1.62; //1.62


/*******************************************************
 *
 * Parameters for synapse using GABA receptors
 *
 *******************************************************/

/**
 *
 * Decay time of GABA receptors (Second)
 *
 **/
const double gabaDecayTime = 0.005; //0.005

/**
 *
 * Reference potential of GABA receptor (mV)
 *
 **/
const double gabaReferencePotential = -70.0;//-70

/**
 *
 * Leak conductance for GABA receptor with pyramidal cell
 *
 **/
const double gabaPyramidalCellLeakConductance = 1.25; //2.5

/**
 *
 * Leak conductance for GABA receptor with interneurons
 *
 **/
const double gabaInterneuronsLeakConductance = 0.973; //2


/*******************************************************
 *
 * Parameters for synapse using NMDA receptors
 *
 *******************************************************/

/**
 *
 * Decay time of NMDA receptors (Second)
 *
 **/
const double nmdaDecayTime = 0.050;//0.100;

/**
 *
 * Alpha frequency of NMDA receptors (ms-1)
 *
 **/
const double nmdaAlpha = 500; // normally 500


/**
 *
 * Decay time of NMDA receptors (Second)
 *
 **/
const double nmdaRiseTime = 0.002;

/**
 *
 * Reference potential of NMDA receptor (mV)
 *
 **/
const double nmdaReferencePotential = 0;

/**
 *
 * Leak conductance for NMDA receptor with pyramidal cell
 *
 **/
const double nmdaPyramidalCellLeakConductance = 0.327; //0.5

/**
 *
 * Leak conductance for NMDA receptor with interneurons
 *
 **/
const double nmdaInterneuronsLeakConductance = 0.258; //0.4


#endif

//) parameters.hpp
