//( simulationtest.hpp

#ifndef SIMULATIONTEST_HPP
#define SIMULATIONTEST_HPP

#include <sys/types.h>
#include <unistd.h>

#include <string>
#include <iostream>
#include <fstream>

using namespace std;


#include "timedependantobject.hpp"
#include "globalVar.h"


//( SimulationTest

/**
 * New simulation
 *
 * @author Thomas
 **/
class SimulationTest :
  public TimeDependantObject
{
public:
  /**
   *
   * Constructor
   *
   * @param   simulationTime  time of the simulation
   *
   *
   **/
  SimulationTest(const double simulationTime);

  /**
   * Default destructor
   *
   **/
  ~SimulationTest();


public:
  /**
   *
   * Run the simulation
   *
   *
   * @return  void
   *
   **/
  virtual void run();


public:
  /**
   *
   * Start the program gnuplot
   *
   * @param   gnuplotScript  script used to update the graphics
   *
   * @return  void
   *
   **/
  void startGnuplot(const string & gnuplotScript);


protected:
  /**
   *
   * Set the conditions for a step
   *
   *
   * @return  void
   *
   **/
  virtual void setConditions() = 0;

  /**
   *
   * Print the output of the simulation
   *
   * @return  void
   *
   **/
  virtual void printData() = 0;


private:
  /**
   *
   * Print the time on the standard output
   *
   *
   * @return  void
   *
   **/
  void printTime();


protected:
  /**
   *
   * Total number of step to run the complete simulation
   *
   **/
  int numberOfStep;


private:
  /**
   *
   * File descriptor for the pipe to communicate with gnuplot
   *
   **/
  int filedescriptor[2];

  /**
   *
   * Pid for the gnuplot program
   *
   **/
  pid_t gnuplotPID;

  /**
   *
   * Script used by gnuplot to render the result of the computation
   *
   **/
  string gnuplotScript;

  /**
   *
   * Buffer used to save chrono value
   *
   **/
  double lastChrono;

}
;

//) SimulationTest


#endif

//) simulationtest.hpp
