//( spiketimemetric.cpp



#include "spiketimemetric.hpp"


double spikeTimeMetric(SpikeList * spikes1, SpikeList * spikes2, double cost)
{
    int nbSpikes1=spikes1->size();
    int nbSpikes2=spikes2->size();

    if (cost==0)
    {
        return abs((double)nbSpikes1-nbSpikes2);
    }
//    if (cost==Inf)
//     return(nspi+nspj);

    vector<vector<double> > mat;

    vector<double> buf;
    for (int j=0;j<nbSpikes2+1;j++)
    {
        buf.push_back(0);
    }

    for (int i=0;i<nbSpikes1+1;i++)
    {
        mat.push_back(buf);
    }

    for (int i=0;i<nbSpikes1+1;i++)
    {
        mat[i][0]=i;
    }

    for (int j=0;j<nbSpikes2+1;j++)
    {
        mat[0][j]=j;
    }

    list<double>::iterator ite1=spikes1->begin();
    list<double>::iterator ite2;


    for (int i=1;i<nbSpikes1+1;i++)
    {
        ite2=spikes2->begin();
        for (int j=1;j<nbSpikes2+1;j++)
        {
            mat[i][j]=min(mat[i-1][j]+1,min(mat[i][j-1]+1, mat[i-1][j-1]+cost*fabs( (*ite1)-(*ite2)) ));

            ite2++;
        }
        ite1++;
    }
    //cout<< "spikeTimeMetric ends"<<endl;
    return mat[nbSpikes1][nbSpikes2];
}

double spikeTimeMetricPopulation(PopulationResponse * spikesVector1,PopulationResponse * spikesVector2,double cost)
{

    //cout<< "spikeTimeMetricPopulation"<<endl;
    unsigned int n=spikesVector1->size();
    if (n!=spikesVector2->size())
    {
        cout << "Erreur dans les tailles des populations" << endl;
    }

    vector<double> distances;

    for (unsigned int i=0;i<n;i++)
    {
        distances.push_back( spikeTimeMetric(&((*spikesVector1)[i]),&((*spikesVector2)[i]),cost) );
    }

    double result=0;

    for(vector<double>::iterator ite=distances.begin(); ite!= distances.end();ite++)
    {
        //result=max(result,(*ite));
        //result+=(*ite);
        result+=(*ite)*(*ite);
    }

    //return result;
    return sqrt(result);
}

void testSpikeTimeMetric()
{
     list<double> sp1;
     sp1.push_back(1.0);
     sp1.push_back(2.0);
     sp1.push_back(3.0);
     sp1.push_back(4.0);


     list<double> sp2;
     sp2.push_back(2.5);
     sp2.push_back(3.5);


    for(int cost=0;cost< 100; cost++)
    {
        cout <<(double)cost/10.0 << "\t" << spikeTimeMetric(&sp1,&sp2,(double)cost/10.0) <<endl;
    }
}

double evaluateInterStimuliDistances(vector<PopulationResponse*> * pop1,vector<PopulationResponse*> * pop2,double cost)
{
    double minD, maxD, meanD,buf;
    minD=3e100;
    maxD=0;
    meanD=0;

    //cout << "Distances test1-test2" << endl;
    for (int i=0;i<10;i++)
    {
        for (int j=0;j<10;j++)
        {
            //cout << i << " et " << j << "\t"<< spikeTimeMetricPopulation(&listOfResult1[i],&listOfResult2[j],50) << endl;
            buf=spikeTimeMetricPopulation((*pop1)[i],(*pop2)[j],cost);
            minD=min(minD,buf);
            maxD=max(maxD,buf);
            meanD+=buf;
        }
    }
    //cout <<"Min:" <<minD << endl;
    //cout <<"Max:" <<maxD << endl;
    //cout <<"Mean:" <<meanD/100.0 << endl;

    return minD;
}

double evaluateIntraStimulusDistances(vector<PopulationResponse*> * pop,double cost)
{
//    unsigned int n= pop->size();

    double minD, maxD, meanD,buf;
    minD=3e100;
    maxD=0;
    meanD=0;

    //cout << "Distances internes" << endl;
    for (int i=0;i<10;i++)
    {
        for (int j=i+1;j<10;j++)
        {
            //cout << i << " et " << j << "\t"<< spikeTimeMetricPopulation(&listOfResult1[i],&listOfResult1[j],50) << endl;
            buf=spikeTimeMetricPopulation((*pop)[i],(*pop)[j],cost);
            minD=min(minD,buf);
            maxD=max(maxD,buf);
            meanD+=buf;
        }
    }
    //cout <<"Min:" <<minD << endl;
    //cout <<"Max:" <<maxD << endl;
    //cout <<"Mean:" <<2*meanD/(n*(n+1)) << endl;

    return maxD;
}

//) spiketimemetric.cpp
