//( synapseampa.hpp

#ifndef SYNAPSEAMPA_HPP
#define SYNAPSEAMPA_HPP


#include "synapse.hpp"


//( SynapseAMPA

/**
 * Synapse using AMPA receptors
 *
 * @author Thomas
 **/
class SynapseAMPA :
  public Synapse
{
public:
  /**
   *
   * Constructor of the Synapse
   * 
   *
   **/
  SynapseAMPA();

  /**
   * Default destructor
   *
   **/
  ~SynapseAMPA();


public:
  /**
   *
   *
   * @return  the reference potential of the receptors
   *
   **/
  double getReceptorReferencePotential() const;

  /**
   *
   *
   * @param   postsynapticNeuron  the postsynaptic neuron of this synapse
   *
   * @return  the leak conductance of the receptors
   *
   **/
  double getReceptorLeakConductance(const LifNeuron * postsynapticNeuron) const;


protected:
  /**
   *
   * Create a new synapse of the same model 
   * 
   *
   * @return  the newly created synapse
   *
   **/
  Synapse * createNewSynapse() const;


protected:
  /**
   *
   * Compute the derivate of the equation modeling the behaviour of
   * this neurons
   * 
   * @param   currentValue  the last value of the function
   * @param   rungeKutta    indicate the step of the algorithm of Runge Kutta
   *
   * @return  the result of the computation
   *
   **/
  double computeDerivate(double currentValue, RungeKutta rungeKutta);
}
;

//) SynapseAMPA


#endif

//) synapseampa.hpp
