//( testafferents.hpp

#ifndef TESTAFFERENTS_HPP
#define TESTAFFERENTS_HPP

#include "simulationtest.hpp"
#include "directafferents.hpp"
#include <vector>
#include <list>

//( TestAfferents

/**
 * class for testing the afferents
 *
 * @author Xavier
**/
class TestAfferents:
        public SimulationTest
{
public:
    /**
   *
   * Constructor
   *
   * @param   simulationTime  time of the simulation
   * @param   datafile        file to save the output of the simulation (raster)
   *
   **/
    
    
    TestAfferents(          const string & datafile,
                            const unsigned int nbOfAfferents,
                            const double simulationTime=10.0) ;
    
    /**
   * Default destructor
   *
   **/
    ~TestAfferents();
    
    
public:
    /**
   *
   * Compute one more step for this object
   *
   *
   * @return  void
   *
   **/
    void computeOneStep();
    
    /**
   *
   * Set the conditions for a step
   *
   *
   * @return  void
   *
   **/
    void setConditions();
    
    /**
   *
   * Print the output of the simulation
   *
   * @return  void
   *
   **/
    void printData();
    
    /**
   *
   * updates the Temporal DataList
   *
   * @return  void
   *
   **/
    void setTemporalDataList(vector<vector<double> > * tempDataList);
    
protected:
    /**
   *
   * Attractor network used for the simulation
   *
   **/
    DirectAfferents directAfferents;
    
    /**
   *
   * File used to get the inputs from the artificial skin
   *
   **/
    ifstream inputFile;
    
    
    /**
   *
   * File used to write the data of the simulation
   *
   **/
    ofstream outputFile;
    
    
    // contains the input of all neuron at each timestep
    vector<vector<double> > * temporalDataList;

    // counters used to estimate spike times
    int timeIncrement;
//    boost::posix_time::ptime preciseTime2;
    int acq_period;
public:
    void resetSpikeTimer();
}
;

//) TestAfferents


#endif

//) testafferents.hpp
