//( cd_socket.hpp

#ifndef CD_SOCKET_HPP
#define CD_SOCKET_HPP

//socket includes
#ifdef WIN32 
#include <windows.h>
typedef DWORD socklen_t;

#else
#include <sys/types.h>
#include <sys/socket.h> 
#include <sys/un.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <fcntl.h> // to set special server flags
#include <unistd.h> // misc symbolic constants and types
#include <netdb.h>
#include <netinet/tcp.h>
#define closesocket close
#endif

#include <string>
#include <fstream>

using namespace std;

#include "communicationdevice.hpp"

#define CLIENT 0
#define SERVER 1

#define UNIX_FAMILY 0
#define INET_FAMILY 1

//( Cd_Socket

/**
 * A socket used to communicate with another program
 *
 * @author Christian Boucheny
 **/
class Cd_Socket :
  public CommunicationDevice
{
public:
  /**
   * Default constructor. The server address is set to local (127.0.0.1), the tcp_port can be defined here
   * 
   * @param status CLIENT or SERVER 
   * @param family UNIX_FAMILY or INET_FAMILY 
   * @param tcp_port server tcp port to be used 
   *
   **/
  Cd_Socket(unsigned short status=CLIENT , unsigned short family=UNIX_FAMILY, unsigned short tcp_port=10000);

  /**
   *
   * Constructor for which parameters of the tcp connection are stored in an external file 
   * 
   * @param status CLIENT or SERVER 
   * @param family UNIX_FAMILY or INET_FAMILY 
   * @param server_address address of the server host
   * @param tcp_port tcp_port to connect
   *
   **/
  Cd_Socket(unsigned short status, unsigned short family, string server_address,unsigned short tcp_port);

  /**
   * Default destructor
   *
   **/
  ~Cd_Socket();

  /**
   *
   * Send a block of data
   * 
   * @param   buffer       data to send. Need a cast to char*
   * @param   buffer_size  size of the packet
   *
   * @return  the error signal 
   *
   **/
  int sendBuffer(void* buffer,int buffer_size);

  /**
   *
   * Receive a block of data
   * 
   * @param   buffer       data to send
   * @param   buffer_size  size of the packet
   *
   * @return  the error signal 
   *
   **/
  int receiveBuffer(void* buffer,int buffer_size);


protected:
  /**
   *
   * initialize the socket connections with parameters defined in constructor 
   * 
   *
   **/
  void initializeSocket();

  /**
   *
   * The socket itself
   * 
   **/
  int socket_fd;

  /**
   *
   * host address of the server
   * 
   **/
  string serv_host_addr;

  /**
   *
   * tcp port of the server
   * 
   **/
  unsigned short serv_tcp_port;

  /**
   *
   * is it a client or server ? 0 = CLIENT / 1 = SERVER
   * 
   **/
  unsigned short status;

  /**
   *
   * Describes the family of the socket. 0 = UNIX_FAMILY (AF_UNIX) / 1 = INET_FAMILY (AF_INET)
   * 
   **/
  unsigned short family;
  
  /**
   *
   * return value of called I-O functions 
   * 
   **/
  int ret_status;
}
;

//) Cd_Socket


#endif

//) cd_socket.hpp
